@@ -97,8 +97,7 @@
 int main(
 
   int    argc,
-  char **argv,
-  char **envp)
+  char **argv)
 
   {
   int c;
