@@ -105,6 +105,8 @@
 #include <signal.h>
 #include <termios.h>
 #include <unistd.h>
+#include <stdlib.h>
+#include <assert.h>
 
 #ifdef sun
 #include <sys/stream.h>
@@ -114,7 +116,7 @@
 #include <sys/ioctl.h>
 #endif	/* HAVE_SYS_IOCTL_H */
 
-#if !defined(sgi) && !defined(_AIX) && !defined(linux)
+#if defined(HAVE_SYS_TTY_H)
 #include <sys/tty.h>
 #endif
 
@@ -125,9 +127,11 @@
 
 #include "cmds.h"
 #include "net_connect.h"
-#include "pbs_version.h"
+#include "log.h"
+#include "port_forwarding.h"
 
 static char *DefaultFilterPath = "/usr/local/sbin/torque_submitfilter";
+static char *DefaultXauthPath = XAUTH_PATH;
 
 #define SUBMIT_FILTER_ADMIN_REJECT_CODE -1 
 
@@ -139,10 +143,107 @@ char PBS_Filter[256];
 char PBS_InitDir[256];
 char PBS_RootDir[256];
 
+char xauth_path[256];
+
+int interactivechild=0;
+int x11child=0;
 
 int do_dir(char *);
 int process_opts(int,char **,int);
 
+/* adapted from openssh */
+static char *
+x11_get_proto(void)
+{       
+        char line[512];
+        char proto[512], data[512], screen[512];
+        char *authstring;
+        FILE *f;
+        int got_data = 0;
+        char *display, *p;
+        struct stat st;
+
+        proto[0] = data[0] = screen[0] = '\0';
+
+        if ((display = getenv("DISPLAY")) == NULL)
+          {
+          fprintf(stderr,"qsub: DISPLAY not set\n");
+          return (NULL);
+          }
+
+        if (stat(xauth_path,&st))
+          {
+          perror("qsub: xauth: ");
+          return (NULL);
+          }
+
+        /* Try to get Xauthority information for the display. */
+        if (strncmp(display, "localhost:", 10) == 0)
+                /*
+                 * Handle FamilyLocal case where $DISPLAY does
+                 * not match an authorization entry.  For this we
+                 * just try "xauth list unix:displaynum.screennum".
+                 * XXX: "localhost" match to determine FamilyLocal
+                 *      is not perfect.
+                 */
+                snprintf(line, sizeof line, "%s list unix:%s 2>/dev/null",
+                    xauth_path,display+10);
+        else    
+                snprintf(line, sizeof line, "%s list %.200s 2>/dev/null",
+                    xauth_path,display);
+
+	p = strchr(display,':');
+        if (p)
+          p = strchr(p,'.');
+        if (p)
+          strncpy(screen,p+1,512);
+        else
+          strcpy(screen,"0");
+
+        if (getenv("PBSDEBUG") != NULL)
+          fprintf(stderr,"x11_get_proto: %s\n", line);
+
+        f = popen(line, "r");
+        if (f && fgets(line, sizeof(line), f) &&
+            sscanf(line, "%*s %511s %511s", proto, data) == 2)
+                got_data = 1;
+        if (f)  
+                pclose(f);
+
+#if 0 /* we aren't inspecting the returned xauth data yet */
+        /*
+         * If we didn't get authentication data, just make up some
+         * data.  The forwarding code will check the validity of the
+         * response anyway, and substitute this data.  The X11
+         * server, however, will ignore this fake data and use
+         * whatever authentication mechanisms it was using otherwise
+         * for the local connection.
+         */
+        if (!got_data) {
+                u_int32_t _rand = 0;                                                         
+                int i;
+
+                fprintf(stderr,"Warning: No xauth data; using fake authentication data for X11 forwarding.\n");
+                strncpy(proto, "MIT-MAGIC-COOKIE-1", sizeof proto);
+                for (i = 0; i < 16; i++) {
+                        if (i % 4 == 0)
+                                _rand = rand();
+                        snprintf(data + 2 * i, sizeof data - 2 * i, "%02x", _rand & 0xff);
+                        _rand >>= 8;
+                }
+        }
+#endif
+
+    if (!got_data)
+      return(NULL);
+
+    authstring = malloc(strlen(proto) + strlen(data) + strlen(screen) +4);
+    sprintf(authstring,"%s:%s:%s",proto, data, screen);
+
+    return(authstring);
+
+}
+
 
 
 char *set_dir_prefix(
@@ -228,6 +329,58 @@ char *ispbsdir(
 
 
 
+#define MMAX_VERIFY_BYTES 50
+
+int istext(
+
+  FILE   *fd,      /* I */
+  int    *IsText)  /* O (optional) */
+
+  {
+  int i;
+  int c;
+
+  if (IsText != NULL)
+    *IsText = FALSE;
+
+  if (fd == NULL)
+    {
+    return(0);
+    }
+
+  if (fd == stdin)
+    {
+    return(1);
+    }
+
+  /* read first characters to ensure this is ASCII text */
+
+  for(i = 0; i < MMAX_VERIFY_BYTES; i++)
+    {
+    c = fgetc(fd);
+
+    if (c == EOF)
+      break;
+
+    if (!isprint(c) && !isspace(c))
+      {
+      fseek(fd,0,SEEK_SET);
+
+      return(0);
+      }
+    }  /* END for(i) */
+
+  if (IsText != NULL)
+    *IsText = TRUE;
+
+  fseek(fd,0,SEEK_SET);
+
+  return(1);
+  }  /* END FileIsText() */
+
+
+
+
 
 int get_script(
 
@@ -263,6 +416,16 @@ int get_script(
 
   /*  If the submitfilter exists, run it.                               */
 
+  /* check that the file is text */
+
+  if (istext(file,NULL) == 0)
+    {
+    fprintf(stderr,
+      "qsub:  file must be an ascii script\n");
+
+    return(4);
+    }
+
   if (stat(PBS_Filter,&sfilter) != -1) 
     {
     /* Create a copy of the script to run through the submit filter. */
@@ -504,11 +667,19 @@ void make_argv(
   char *line)
 
   {
-  char *l, *b, *c;
-  char buffer[4096];
+  char *l, *b, *c, *buffer;
   int len;
   char quote;
 
+  buffer = malloc(strlen(line) + 1);
+
+  if (buffer == NULL)
+    {
+    fprintf(stderr, "qsub: out of memory\n");
+
+    exit(2);
+    }
+
   *argc = 0;
 
   argv[(*argc)++] = "qsub";
@@ -551,6 +722,8 @@ void make_argv(
       {
       len = c - l;
 
+      assert(len > 0);
+
       if (argv[*argc] != NULL) 
         free(argv[*argc]);
 
@@ -584,6 +757,8 @@ void make_argv(
     {
     len = c - l;
 
+    assert(len > 0);
+
     if (argv[*argc] != NULL) 
       free(argv[*argc]);
 
@@ -600,6 +775,8 @@ void make_argv(
     strcpy(argv[(*argc)++],buffer);
     }
 
+  free(buffer);
+
   return;
   }  /* END make_argv() */
 
@@ -636,7 +813,6 @@ int do_dir(
 
 int inter_sock;
 struct termios oldtio;
-struct winsize wsz;
 
 struct attrl *attrib = NULL;
 char *new_jobname;                  /* return from submit request */
@@ -645,9 +821,11 @@ char path_out[MAXPATHLEN + 1];
 char destination[PBS_MAXDEST];
 static char server_out[PBS_MAXSERVERNAME + PBS_MAXPORTNUM + 2];
 char server_host[PBS_MAXHOSTNAME + 1];
+long cnt2server_retry=-100;
 
 /* state booleans for protecting already-set options */
 int a_opt = FALSE;
+int b_opt = FALSE;
 int c_opt = FALSE;
 int e_opt = FALSE;
 int h_opt = FALSE;
@@ -673,6 +851,8 @@ int Interact_opt  = FALSE;
 int Stagein_opt   = FALSE;
 int Stageout_opt  = FALSE;
 int Grouplist_opt = FALSE;
+int Forwardx11_opt = FALSE;
+int Umask_opt = FALSE;
 char *v_value = NULL;
 
 
@@ -783,7 +963,6 @@ int set_job_env(
   char *s, *c, *env, l;
   unsigned   len;
   int   rc;
-  char *getcwd();
 
   int   eindex;
 
@@ -1209,10 +1388,10 @@ final:
  *	will be connected to this socket.
  */
 
-char *interactive_port()
+char *interactive_port(int *sock)
 
   {
-  unsigned int namelen;
+  torque_socklen_t namelen;
   static char portstring[8];
   struct sockaddr_in myaddr;
   unsigned short port;
@@ -1223,20 +1402,21 @@ char *interactive_port()
       exit(1);
     }
 
-  inter_sock = socket(AF_INET, SOCK_STREAM, 0);
+  *sock = socket(AF_INET, SOCK_STREAM, 0);
 
-  if (inter_sock < 0) 
+  if (*sock < 0) 
     {
     perror("qsub: unable to obtain socket");
 
     exit(1);
     }
 
+  namelen = sizeof(myaddr);
   myaddr.sin_family = AF_INET;
   myaddr.sin_addr.s_addr = INADDR_ANY;
   myaddr.sin_port = 0;
 
-  if (bind(inter_sock,(struct sockaddr *)&myaddr,sizeof(myaddr)) < 0) 
+  if (bind(*sock,(struct sockaddr *)&myaddr,namelen) < 0) 
     {
     perror("qsub: unable to bind to socket");
 
@@ -1245,9 +1425,7 @@ char *interactive_port()
 
   /* get port number assigned */
 
-  namelen = sizeof(myaddr);
-
-  if (getsockname(inter_sock,(struct sockaddr *)&myaddr,&namelen) < 0) 
+  if (getsockname(*sock,(struct sockaddr *)&myaddr,&namelen) < 0) 
     {
     perror("qsub: unable to get port number");
 
@@ -1259,7 +1437,7 @@ char *interactive_port()
   sprintf(portstring,"%u", 
     (unsigned int)port);
 
-  if (listen(inter_sock,1) < 0) 
+  if (listen(*sock,1) < 0) 
     {
     perror("qsub: listen on interactive socket");
 
@@ -1340,7 +1518,8 @@ void stopme(
 
 int reader(
 
-  int s)	/* socket */
+  int s,	/* reading socket */
+  int d)        /* writing socket */
 
   {
   char buf[4096];
@@ -1348,7 +1527,7 @@ int reader(
   char *p;
   int  wc;
 
-  /* read from the socket, and write to stdout */
+  /* read from the socket, and write to d */
 
   /* NOTE:  s should be blocking */
 
@@ -1362,7 +1541,7 @@ int reader(
 
       while (c) 
         {
-        if ((wc = write(1,p,c)) < 0) 
+        if ((wc = write(d,p,c)) < 0) 
           {
           if (errno == EINTR) 
             {
@@ -1416,7 +1595,8 @@ int reader(
 
 void writer(
 
-  int s)  /* socket */
+  int s,  /* writing socket */
+  int d)  /* reader socket */
 
   {
   char c;
@@ -1429,7 +1609,7 @@ void writer(
 
   while (1) 
     {
-    i = read(0,&c,1);
+    i = read(d,&c,1);
 
     if (i > 0) 
       {  
@@ -1443,7 +1623,7 @@ void writer(
 
           /* read next character to check */
 
-          while ((i = read(0,&c,1)) != 1) 
+          while ((i = read(d,&c,1)) != 1) 
             {
             if ((i == -1) && (errno == EINTR))
               continue;
@@ -1558,10 +1738,10 @@ void writer(
 
 int getwinsize(
 
-  struct winsize *pwsz)
+  struct winsize *wsz)
 
   {
-  if (ioctl(0,TIOCGWINSZ,&wsz) < 0) 
+  if (ioctl(0,TIOCGWINSZ,wsz) < 0) 
     {
     perror("qsub: unable to get window size");
 
@@ -1581,18 +1761,24 @@ int getwinsize(
 
 void send_winsize(
 
-  int sock)
+  int sock,
+  struct winsize *wsz)
 
   {
   char  buf[PBS_TERM_BUF_SZ];
 
   sprintf(buf,"WINSIZE %hu,%hu,%hu,%hu", 
-    wsz.ws_row, 
-    wsz.ws_col,
-    wsz.ws_xpixel, 
-    wsz.ws_ypixel);
+    wsz->ws_row, 
+    wsz->ws_col,
+    wsz->ws_xpixel, 
+    wsz->ws_ypixel);
 
-  write(sock,buf,PBS_TERM_BUF_SZ);
+  if (write(sock,buf,PBS_TERM_BUF_SZ) != PBS_TERM_BUF_SZ)
+    {
+    perror("sending winsize");
+
+    exit(2);
+    }
 
   return;
   } 
@@ -1622,7 +1808,12 @@ void send_term(
   else
     strncat(buf,term,PBS_TERM_BUF_SZ - 5);
 
-  write(sock, buf, PBS_TERM_BUF_SZ);
+  if (write(sock, buf, PBS_TERM_BUF_SZ) != PBS_TERM_BUF_SZ)
+    {
+    perror("sending term type");
+
+    exit(2);
+    }
 
   cc_array[0] = oldtio.c_cc[VINTR];
   cc_array[1] = oldtio.c_cc[VQUIT];
@@ -1631,7 +1822,12 @@ void send_term(
   cc_array[4] = oldtio.c_cc[VEOF];
   cc_array[5] = oldtio.c_cc[VSUSP];
 
-  write(sock, cc_array, PBS_TERM_CCA);
+  if (write(sock, cc_array, PBS_TERM_CCA) != PBS_TERM_CCA)
+    {
+    perror("sending term options");
+
+    exit(2);
+    }
 
   return;
   }
@@ -1671,6 +1867,11 @@ void catchchild(
       }
     }
    
+    if (interactivechild > 0)
+      kill(interactivechild,SIGTERM);
+    if (x11child > 0)
+      kill(x11child,SIGTERM);
+
   /* reset terminal to cooked mode */
 
   tcsetattr(0,TCSANOW,&oldtio);
@@ -1800,6 +2001,39 @@ void catchint(
   }  /* END catchint() */
 
 
+void x11handler(int inter_sock)
+  {
+  struct pfwdsock *socks;
+  int n;
+  char *display;
+
+  socks=calloc(sizeof (struct pfwdsock),NUM_SOCKS);
+
+  if (!socks)
+    {
+    perror("x11handler malloc: ");
+
+    exit(EXIT_FAILURE);
+    }
+
+  for (n=0;n<NUM_SOCKS;n++)
+    (socks+n)->active=0;
+
+  (socks+0)->sock = inter_sock;
+  (socks+0)->active=1;
+  (socks+0)->listening=1;
+
+  /* Try to open a socket for the local X server. */
+  display = getenv("DISPLAY");
+  if (!display) {
+    fprintf(stderr,"DISPLAY not set.");
+    return;
+  }
+
+  port_forwarder(socks,x11_connect_display,display,0);
+
+  exit(EXIT_FAILURE);
+}
 
 
 /*
@@ -1811,18 +2045,17 @@ void interactive()
   {
   int  amt;
   char cur_server[PBS_MAXSERVERNAME + PBS_MAXPORTNUM + 2];
-  unsigned int fromlen;
 
-  char momjobid[PBS_MAXSVRJOBID+1];
+  char momjobid[LOG_BUF_SIZE+1];
   int  news;
   int  nsel;
   char *pc;
   fd_set selset;
   struct sigaction act;
   struct sockaddr_in from;
+  torque_socklen_t fromlen;
   struct timeval timeout;
   struct winsize wsz;
-  int child;
     
   /* Catch SIGINT and SIGTERM, and */
   /* setup to catch Death of child */
@@ -1926,7 +2159,7 @@ void interactive()
 
   /* When MOM connects, she will send the job id for us to verify */
 
-  amt = PBS_MAXSVRJOBID + 1;
+  amt = LOG_BUF_SIZE + 1;
 
   pc = momjobid;
 
@@ -1945,6 +2178,15 @@ void interactive()
     amt -= fromlen;
     }
 
+  if (strncmp(momjobid,"PBS:",4) == 0) 
+    {
+    fprintf(stderr,"qsub: %s\n",momjobid);
+
+    shutdown(news,2);
+
+    exit(1);
+    }
+
   if (strncmp(momjobid,new_jobname,PBS_MAXSVRJOBID) != 0) 
     {
     fprintf(stderr, "qsub: invalid job name from execution server\n");
@@ -1961,14 +2203,14 @@ void interactive()
    */
 
   send_term(news);
-  send_winsize(news);
+  send_winsize(news,&wsz);
 
   printf("qsub: job %s ready\n\n", 
     new_jobname);
 
-    /* set SIGINT, SIGTERM processing to default */
+    /* set SIGINT, SIGTERM processing to ignore */
 
-  act.sa_handler = SIG_DFL;
+  act.sa_handler = SIG_IGN;
 
   if ((sigaction(SIGINT, &act, (struct sigaction *)0) < 0)  ||
       (sigaction(SIGTERM, &act, (struct sigaction *)0) < 0) ||
@@ -1980,9 +2222,9 @@ void interactive()
     exit(1);
     }
 
-  child = fork();
+  interactivechild = fork();
 
-  if (child == 0) 
+  if (interactivechild == 0) 
     {
     /*
      * child process - start the reader function
@@ -1991,7 +2233,7 @@ void interactive()
 
     settermraw(&oldtio);
 
-    reader(news);
+    reader(news,fileno(stdout));
 
     /* reset terminal */
 
@@ -2002,7 +2244,7 @@ void interactive()
 
     exit(0);
     } 
-  else if (child > 0) 
+  else if (interactivechild > 0) 
     {
     /* parent - start the writer function */
 
@@ -2013,12 +2255,29 @@ void interactive()
       exit(1);
       }
 
-    writer(news);
+    if (Forwardx11_opt)
+      {
+      if ((x11child=fork()) == 0)
+        {
+        act.sa_handler = SIG_DFL;
+
+        sigaction(SIGTERM,&act,(struct sigaction *)0);
+
+        x11handler(inter_sock);
+        }
+      }
+
+    writer(news,fileno(stdin));
 
     /* all done - make sure reader child is gone and reset terminal */
 
-    kill(child,SIGTERM);
+    if (interactivechild > 0)
+      kill(interactivechild,SIGTERM);
+    if (x11child > 0)
+      kill(x11child,SIGTERM);
 
+    shutdown(inter_sock,SHUT_RDWR);
+    close(inter_sock);
     tcsetattr(0,TCSANOW,&oldtio);
 
     exit(0);
@@ -2068,13 +2327,11 @@ int process_opts(
   int tmpfd;
 
 #if !defined(PBS_NO_POSIX_VIOLATION)
-#define GETOPT_ARGS "a:A:c:C:d:D:e:hIj:k:l:m:M:N:o:p:q:r:S:u:v:VW:z-:"
+#define GETOPT_ARGS "a:A:b:c:C:d:D:e:hIj:k:l:m:M:N:o:p:q:r:S:u:v:VW:Xz-:"
 #else
 #define GETOPT_ARGS "a:A:c:C:e:hj:k:l:m:M:N:o:p:q:r:S:u:v:VW:z"
 #endif	/* PBS_NO_POSIX_VIOLATION */
 
-#define MAX_RES_LIST_LEN 64  /* doesn't seem to be used? */
-
 /* The following macro, together the value of passet (pass + 1) is used	*/
 /* to enforce the following rules: 1. option on the command line take	*/
 /* precedence over those in script directives.   2. With in the command	*/
@@ -2085,7 +2342,7 @@ int process_opts(
 
   passet = pass + 1;
 
-  if ( pass > 0 ) 
+  if (pass > 0) 
     {
 #ifdef linux
     optind = 0;  /* prime getopt's starting point */
@@ -2103,7 +2360,7 @@ int process_opts(
         if ((optarg != NULL) && !strcmp(optarg,"version"))
           {
           fprintf(stderr,"version: %s\n",
-            PBS_VERSION);
+            PACKAGE_VERSION);
 
           exit(0);
           }
@@ -2159,6 +2416,17 @@ int process_opts(
 
         break;
 
+      case 'b':
+
+        if_cmd_line(b_opt) 
+          {
+          b_opt = passet;
+
+          cnt2server_retry=atoi(optarg);
+          }
+
+        break;
+
       case 'c':
 
         if_cmd_line(c_opt) 
@@ -2371,7 +2639,7 @@ int process_opts(
           {
           Interact_opt = passet;
 
-          set_attr(&attrib,ATTR_inter,interactive_port());
+          set_attr(&attrib,ATTR_inter,interactive_port(&inter_sock));
           }
 
         break;
@@ -2859,9 +3127,14 @@ int process_opts(
                 break;
                 }
 
-              set_attr(&attrib,ATTR_inter,interactive_port());
+              set_attr(&attrib,ATTR_inter,interactive_port(&inter_sock));
               }
             } 
+          else if (!strcmp(keyword,ATTR_umask))
+            {
+            Umask_opt = passet;
+            set_attr(&attrib,ATTR_umask,valuewd);
+            }
           else 
             {
             /* generic job attribute specified */
@@ -2881,6 +3154,22 @@ int process_opts(
 
         break;
 
+      case 'X':
+
+        if_cmd_line(Forwardx11_opt) 
+          {
+          Forwardx11_opt = passet;
+
+          if (!getenv("DISPLAY"))
+            {
+            fprintf(stderr,"qsub: DISPLAY not set\n");
+
+            errflg++;
+            }
+          }
+
+        break;
+
       case 'z':
  
         if_cmd_line(z_opt) 
@@ -3091,8 +3380,12 @@ int load_config(
 
   int  length = strlen(PBS_SERVER_HOME) + strlen(TCONST_CFGFILE) + 1;
 
+  char *ptr;
+
   if (length >= MAXPATHLEN)
     {
+    /* FAILURE */
+
     return(1);
     }
 
@@ -3106,14 +3399,32 @@ int load_config(
 
   if ((config_stream = fopen(home_dir,"r")) == NULL)
     {
+    /* FAILURE */
+
     return(1);
     }
 
   if ((fread(config_buf,BufSize,1,config_stream) <= 0) && (ferror(config_stream) != 0))
     {
+    /* FAILURE */
+
     return(1);
     }
 
+  ptr = config_buf;
+
+  while ((ptr = strchr(ptr,'#')) != NULL)
+    {
+    ptr++;
+
+    for (;(*ptr != '\0') && (*ptr != '\n');ptr++)
+      {
+      *ptr = ' ';
+      }
+    }   /* END while ((ptr = strchr(ptr,'#')) != NULL) */ 
+
+  /* SUCCESS */
+
   return(0);
   }  /* END load_config() */
 
@@ -3157,6 +3468,7 @@ char *get_param(
 
 
 
+
 /* qsub main */
 
 int main( 
@@ -3169,7 +3481,6 @@ int main(
   int errflg;                         /* option error */
   static char script[MAXPATHLEN + 1] = ""; /* name of script file */
   char script_tmp[MAXPATHLEN + 1] = "";    /* name of script file copy */
-  char  basename[16];			/* base name of script for job name*/
   char *bnp;
   FILE *f;                            /* FILE pointer to the script */
   char *q_n_out;                      /* queue part of destination */
@@ -3183,17 +3494,25 @@ int main(
   char config_buf[MAX_LINE_LEN];      /* Buffer holds config file */
   char *param_val;                    /* value of parameter returned from config */
 
+  char *submit_args_str;              /* buffer to hold args */
+  int   argi, argslen = 0;
+
+  if ((param_val=getenv("PBS_CLIENTRETRY")) != NULL)
+    {
+    cnt2server_retry=atoi(param_val);
+    }
 
   errflg = process_opts(argc,argv,0);  /* get cmd-line options */
 
   if (errflg || ((optind + 1) < argc)) 
     {
     static char usage[] =
-"usage: qsub [-a date_time] [-A account_string] [-c { c[=<INTERVAL>] | s | n }]\n\
-[-C directive_prefix] [-d path] --D path] [-e path] [-h] [-I] [-j oe] [-k {oe}]\n\
-[-l resource_list] [-m {abe}] [-M user_list] [-N jobname] [-o path]\n\
-[-p priority] [-q queue] [-r y|n] [-S path] [-u user_list]\n\
-[-W otherattributes=value...] [-v variable_list] [-V ] [-z] [script]\n";
+"usage: qsub [-a date_time] [-A account_string] [-b secs]\n\
+[-c { c[=<INTERVAL>] | s | n }] [-C directive_prefix] [-d path] [-D path]\n\
+[-e path] [-h] [-I] [-j oe] [-k {oe}] [-l resource_list] [-m {abe}]\n\
+[-M user_list] [-N jobname] [-o path] [-p priority] [-q queue] [-r y|n]\n\
+[-S path] [-u user_list] [-X] [-W otherattributes=value...] [-v variable_list]\n\
+[-V ] [-z] [script]\n";
 
     fprintf(stderr,usage);
 
@@ -3203,8 +3522,12 @@ int main(
   /* check TORQUE config settings */
 
   strcpy(PBS_Filter,DefaultFilterPath);
+  strcpy(xauth_path,DefaultXauthPath);
   server_host[0] = '\0';
 
+  if (getenv("PBSDEBUG") != NULL)
+    fprintf(stderr,"xauth_path=%s\n",xauth_path);
+
   if (load_config(config_buf,sizeof(config_buf)) == 0)
     {
     if ((param_val = get_param("QSUBSLEEP",config_buf)) != NULL)
@@ -3223,11 +3546,80 @@ int main(
       strncpy(server_host,param_val,sizeof(server_host));
       server_host[sizeof(server_host) - 1] = '\0';
       }
+
+    if ((param_val = get_param("XAUTHPATH",config_buf)) != NULL)
+      {
+      strncpy(xauth_path,param_val,sizeof(xauth_path));
+      xauth_path[sizeof(xauth_path) - 1] = '\0';
+      }
+
+    if ((param_val = get_param("CLIENTRETRY",config_buf)) != NULL)
+      {
+      if (cnt2server_retry == -100)
+        cnt2server_retry=atoi(param_val);
+      }
     }
 
   if (optind < argc) 
     strcpy(script,argv[optind]);
 
+  /* set the submit_args */
+  for (argi = 1; argi < optind; argi++)
+    {
+    argslen += strlen(argv[argi]) + 1;
+    }
+
+  if (argslen > 0)
+    {
+    submit_args_str = malloc(sizeof(char) * argslen);
+
+    if (submit_args_str == NULL)
+      {
+      fprintf(stderr,"qsub: out of memory\n");
+  
+      exit(2);
+      }
+
+    *submit_args_str = '\0';
+
+    for (argi = 1;argi < optind;argi++)
+      {
+      strcat(submit_args_str,argv[argi]);
+
+      if (argi != optind - 1)
+        {
+        strcat(submit_args_str," ");
+        }
+      }
+
+    set_attr(&attrib,ATTR_submit_args,submit_args_str);
+
+    free(submit_args_str);
+    }
+  /* end setting submit_args */
+
+  if (Forwardx11_opt)
+    {
+    char *x11authstr;
+
+    /* get the DISPLAY's auth proto, data, and screen number */
+    if ((x11authstr = x11_get_proto()) != NULL)
+      {
+
+      /* stuff this info into the job */
+      set_attr(&attrib,ATTR_forwardx11,x11authstr);
+
+      if (getenv("PBSDEBUG") != NULL)
+        fprintf(stderr,"x11auth string: %s\n",x11authstr);
+      }
+    else
+      {
+      fprintf(stderr,"qsub: Failed to get xauth data.\n");
+
+      exit(1);
+      }
+    }
+
   /* if script is empty, get standard input */
 
   if ((strcmp(script,"") == 0) || (strcmp(script,"-") == 0) ) 
@@ -3281,13 +3673,9 @@ int main(
         else
           bnp = script;
 
-        strncpy(basename, bnp, 15);
-
-        basename[15] = '\0';
-
-        if (check_job_name(basename,0) == 0) 
+        if (check_job_name(bnp,0) == 0) 
           {
-          set_attr(&attrib,ATTR_N,basename);
+          set_attr(&attrib,ATTR_N,bnp);
           } 
         else 
           {
@@ -3342,6 +3730,9 @@ int main(
 
   /* Connect to the server */
 
+  if (cnt2server_retry != -100)
+    cnt2server_conf(cnt2server_retry); /* set number of seconds to retry */
+
   connect = cnt2server(server_out);
 
   if (connect <= 0) 
@@ -3437,3 +3828,5 @@ int main(
   }  /* END main() */
 
 
+
+
