@@ -107,7 +107,7 @@ struct depend {
 	short	  dp_numexp;	/* num jobs expected (on or syncct only) */
 	short	  dp_numreg;	/* num jobs registered (syncct only)     */
 	short	  dp_released;	/* This job released to run (syncwith)   */
-	list_head dp_jobs;	/* list of related jobs  (all)           */
+	tlist_head dp_jobs;	/* list of related jobs  (all)           */
 };
 
 /*
@@ -167,12 +167,12 @@ typedef struct	badplace {
 
 #ifdef NGROUPS_MAX
 struct grpcache {
-	int    gc_ngroup;	/* number of active supplementary groups */
-	int    gc_groups[NGROUPS_MAX];
-	char   gc_homedir[1];	/* more space allocated as part of this	 */
-				/* structure following here		 */
-};
-#endif
+  int    gc_ngroup;	/* number of active supplementary groups */
+  int    gc_groups[NGROUPS_MAX];
+  char   gc_homedir[1];	/* more space allocated as part of this	 */
+			/* structure following here		 */
+  };
+#endif /* END NGROUPS_MAX */
 	
 /*
  * Job attributes/resources are maintained in one of two ways.
@@ -254,6 +254,9 @@ enum job_atr {
 	JOB_ATR_altid,		/* alternate job id, for irix6 = array id */
 	JOB_ATR_etime,		/* time job became eligible to run	  */
 	JOB_ATR_exitstat,	/* exit status of job			  */
+	JOB_ATR_forwardx11,
+	JOB_ATR_submit_args,
+        JOB_ATR_umask,
 #include "site_job_attr_enum.h"
 
 	JOB_ATR_UNKN,		/* the special "unknown" type		  */
@@ -261,7 +264,7 @@ enum job_atr {
 };
 
 /*
- * The "definations" for the job attributes are in the following array,
+ * The "definitions" for the job attributes are in the following array,
  * it is also indexed by the JOB_ATR_... enums.
  */
 
@@ -279,7 +282,7 @@ typedef	struct	hnodent {
 	char	       *hn_host;	/* hostname of node */
 	int		hn_stream;	/* stream to MOM on node */
 	int		hn_sister;	/* save error for KILL_JOB event */
-	list_head	hn_events;	/* pointer to list of events */
+	tlist_head	hn_events;	/* pointer to list of events */
   } hnodent;
 
 
@@ -305,6 +308,7 @@ typedef struct	noderes {
 
 
 /* mother superior to sister communication */
+/*   (sync w/TSisterFailure[] in resmom/mom_main.c) */
 
 #define	SISTER_OKAY	0
 #define SISTER_KILLDONE	1000
@@ -401,18 +405,18 @@ struct job {
 	hnodent	       *ji_hosts;	/* ptr to job host management stuff */
 	vnodent	       *ji_vnods;	/* ptr to job vnode management stuff */
 	noderes	       *ji_resources;	/* ptr to array of node resources */
-	list_head       ji_tasks;	/* list of task structs */
+	tlist_head       ji_tasks;	/* list of task structs */
 	tm_node_id	ji_nodekill;	/* set to nodeid requesting job die */
 	int		ji_flags;	/* mom only flags */
 	char	       *ji_globid;	/* global job id */
 	int		ji_stdout;	/* port for stdout */
 	int		ji_stderr;	/* port for stderr */
 #else					/* END MOM ONLY */
-	list_head	ji_svrtask;	/* links to svr work_task list */
+	tlist_head	ji_svrtask;	/* links to svr work_task list */
 	struct pbs_queue  *ji_qhdr;	/* current queue header */
 	int		ji_lastdest;	/* last destin tried by route */
 	int		ji_retryok;	/* ok to retry, some reject was temp */
-	list_head	ji_rejectdest;	/* list of rejected destinations */
+	tlist_head	ji_rejectdest;	/* list of rejected destinations */
 #endif					/* END SERVER ONLY */
 
 	/*
@@ -462,8 +466,8 @@ struct job {
 	 * Its presence is for rapid acces to the attributes.
 	 */
 
-	attribute	ji_wattr[JOB_ATR_LAST]; /* decoded attributes  */
-};
+  attribute ji_wattr[JOB_ATR_LAST]; /* decoded attributes  */
+  };
 
 typedef struct job job;
 
@@ -478,8 +482,8 @@ typedef struct	task {
 	int		ti_fd;		/* DIS file descriptor to task */
 	int		ti_flags;	/* task internal flags */
 	tm_event_t	ti_register;	/* event if task registers */
-	list_head	ti_obits;	/* list of obit events */
-	list_head	ti_info;	/* list of named info */
+	tlist_head	ti_obits;	/* list of obit events */
+	tlist_head	ti_info;	/* list of named info */
 	struct taskfix {
 		char    	ti_parentjobid[PBS_MAXSVRJOBID+1];
 		tm_node_id	ti_parentnode;
@@ -533,19 +537,19 @@ typedef struct	obitent {
 **	available to other tasks in the job.
 */
 typedef struct	infoent {
-	char		*ie_name;	/* published name */
-	void		*ie_info;	/* the glop */
-	size_t		ie_len;		/* how much glop */
-	list_link	ie_next;	/* link to next one */
-} infoent;
+  char		*ie_name;	/* published name */
+  void		*ie_info;	/* the glop */
+  size_t	ie_len;		/* how much glop */
+  list_link	ie_next;	/* link to next one */
+  } infoent;
 
 #define	TI_FLAGS_INIT		1		/* task has called tm_init */
 #define	TI_FLAGS_CHKPT		2		/* task has checkpointed */
 
-#define TI_STATE_EMBRYO		0
-#define	TI_STATE_RUNNING	1
-#define TI_STATE_EXITED		2		/* ti_exitstat valid */
-#define TI_STATE_DEAD		3
+#define TI_STATE_EMBRYO  0
+#define TI_STATE_RUNNING 1              /* includes suspended jobs */
+#define TI_STATE_EXITED	 2		/* ti_exitstat valid */
+#define TI_STATE_DEAD    3
 
 /*
 **      Here is the set of commands for InterMOM (IM) requests.
@@ -710,6 +714,7 @@ task		*task_find	A_((	job		*pjob,
 #define JOB_EXEC_INITRST  -5	/* job aborted on MOM init, chkpt, no migrate */
 #define JOB_EXEC_INITRMG  -6	/* job aborted on MOM init, chkpt, ok migrate */
 #define JOB_EXEC_BADRESRT -7	/* job restart failed */
+#define JOB_EXEC_CMDFAIL  -8	/* exec() of user command failed */
 
 extern void  add_dest A_((job *));
 extern void  depend_clrrdy A_((job *));
@@ -726,31 +731,31 @@ extern job  *job_alloc();
 extern void  job_free A_((job *));
 extern void  job_purge A_((job *));
 extern job  *job_recov A_((char *));
-extern int   job_save A_((job *, int));
-extern int   modify_job_attr A_((job *, svrattrl *, int, int *));
-extern char *prefix_std_file A_((job *, int));
-extern int   set_jobexid A_((job *, attribute *));
-extern int   site_check_user_map A_((job *, char *));
+extern int   job_save A_((job *,int));
+extern int   modify_job_attr A_((job *,svrattrl *,int,int *));
+extern char *prefix_std_file A_((job *,int));
+extern int   set_jobexid A_((job *,attribute *));
+extern int   site_check_user_map A_((job *,char *));
 extern void  svr_dequejob A_((job *));
 extern int   svr_enquejob A_((job *));
-extern void  svr_evaljobstate A_((job *, int *, int *, int));
-extern void  svr_mailowner A_((job *, int mailtype, int force, char *));
-extern void  set_resc_deflt A_((job *));
+extern void  svr_evaljobstate A_((job *,int *,int *,int));
+extern void  svr_mailowner A_((job *,int,int,char *));
+extern void  set_resc_deflt A_((job *,attribute *));
 extern void  set_statechar A_((job *));
-extern int   svr_setjobstate A_((job *, int, int));
+extern int   svr_setjobstate A_((job *,int,int));
 
 #ifdef BATCH_REQUEST_H
 extern job  *chk_job_request A_((char *, struct batch_request *));
 extern int   net_move A_((job *, struct batch_request *));
-extern int   svr_chk_owner A_((struct batch_request *, job *));
-extern struct batch_request *cpy_stage A_((struct batch_request *, job *, enum job_atr, int));
+extern int   svr_chk_owner A_((struct batch_request *,job *));
+extern struct batch_request *cpy_stage A_((struct batch_request *,job *,enum job_atr,int));
 #endif	/* BATCH_REQUEST_H */
 
 #ifdef QUEUE_H
-extern int   svr_chkque A_((job *, pbs_queue *, char *, int mtype));
-extern int   default_router A_((job *, pbs_queue *, long));
-extern int   site_alt_router A_((job *, pbs_queue *, long));
-extern int   site_acl_check A_((job *, pbs_queue *));
+extern int   svr_chkque A_((job *,pbs_queue *,char *,int,char *));
+extern int   default_router A_((job *,pbs_queue *,long));
+extern int   site_alt_router A_((job *,pbs_queue *,long));
+extern int   site_acl_check A_((job *,pbs_queue *));
 #endif	/* QUEUE_H */
 
 #ifdef WORK_TASK_H
