@@ -166,7 +166,7 @@ extern void DIS_rpp_reset A_((void));
 extern void checkret A_((char **,int));
 
 /* defined in mach-dependant mom_mach.c */
-extern int kill_task A_((struct task *,int sig));
+extern int kill_task A_((struct task *,int,int));
 
 /* Defines for pe_io_type, see run_pelog() */
 
