@@ -159,6 +159,9 @@
 #define ATTR_altid	"alt_id"
 #define ATTR_etime	"etime"
 #define ATTR_exitstat	"exit_status"
+#define ATTR_forwardx11 "forward_x11"
+#define ATTR_submit_args "submit_args"
+#define ATTR_umask    "umask"
 
 /* additional queue attributes names */
 
@@ -175,6 +178,7 @@
 #define ATTR_killdelay	"kill_delay"
 #define ATTR_maxgrprun  "max_group_run"
 #define ATTR_maxque	"max_queuable"
+#define ATTR_maxuserque "max_user_queuable"
 #define ATTR_maxuserrun "max_user_run"
 #define ATTR_qtype	"queue_type"
 #define ATTR_rescassn	"resources_assigned"
@@ -199,6 +203,15 @@
 
 /* additional server attributes names */
 
+/* NOTE: steps for adding new attribute described in ??? */
+/*  - create #define ATTR_* in include/pbs_ifl.h
+    - insert SRV_ATR_* in include/server.h
+    - add SRV_ATR_* in include/qmgr_svr_public.h
+    - insert structure in server/svr_attr_def.c
+        NOTE:  structure must be in same relative position as SRV_ATR_*
+    - insert usage code in proper location 
+*/
+
 #define ATTR_aclroot	 "acl_roots"
 #define ATTR_managers	 "managers"
 #define ATTR_dfltque	 "default_queue"
@@ -224,10 +237,17 @@
 #define ATTR_polljobs    "poll_jobs"
 #define ATTR_downonerror "down_on_error"
 #define ATTR_jobnanny    "job_nanny"
+#define ATTR_ownerpurge  "owner_purge"
+#define ATTR_qcqlimits   "queue_centric_limits"
 #define ATTR_momjobsync  "mom_job_sync"
 #define ATTR_maildomain  "mail_domain"
 #define ATTR_pbsversion  "pbs_version"
-
+#define ATTR_submithosts  "submit_hosts"
+#define ATTR_allownodesubmit  "allow_node_submit"
+#define ATTR_autonodenp  "auto_node_np"
+#define ATTR_servername  "server_name"
+#define ATTR_logfilemaxsize "log_file_max_size"
+#define ATTR_logfilerolldepth "log_file_roll_depth"
 
 /* additional node "attributes" names */
 
