@@ -78,7 +78,7 @@
 */
 
 /*
- * @(#) $Id: queue.h,v 1.5 2006/01/10 09:14:29 gstaples Exp $
+ * @(#) $Id: queue.h 1050 2006-10-11 23:48:34Z garrick $
  */
 
 /*
@@ -114,6 +114,7 @@ enum queueattr {
 	QA_ATR_QType,
 	QA_ATR_Priority,
 	QA_ATR_MaxJobs,
+	QA_ATR_MaxUserJobs,
 	QA_ATR_TotalJobs,
 	QA_ATR_JobsByState,
 	QA_ATR_MaxRun,
@@ -129,6 +130,7 @@ enum queueattr {
         QA_ATR_AclGroup,
         QA_ATR_AclLogic,
         QA_ATR_AclGroupSloppy,
+        QA_ATR_MTime,
 
 	/* The following attributes apply only to exection queues */
 
@@ -165,7 +167,7 @@ extern attribute_def que_attr_def[];
  
 struct pbs_queue {
 	list_link	qu_link;		/* forward/backward links */
-	list_head	qu_jobs;		/* jobs in this queue */
+	tlist_head	qu_jobs;		/* jobs in this queue */
 
 	struct queuefix {
 		int	qu_modified;		/* != 0 => update disk file */
@@ -176,6 +178,7 @@ struct pbs_queue {
 	} qu_qs;
 
 	int	qu_numjobs;			/* current numb jobs in queue */
+	int	qu_numcompleted;		/* current numb completed jobs in queue */
 	int	qu_njstate[PBS_NUMJOBSTATE];	/* # of jobs per state */
 	char	qu_jobstbuf[100];
 
