@@ -128,10 +128,22 @@ enum srv_atr {
         SRV_ATR_LogLevel,
         SRV_ATR_DownOnError,
         SRV_ATR_JobNanny,
+        SRV_ATR_OwnerPurge,
+        SRV_ATR_QCQLimits,
         SRV_ATR_MomJobSync,
         SRV_ATR_MailDomain,
 	SRV_ATR_version,
+	SRV_ATR_KillDelay,
+	SRV_ATR_AclLogic,
+	SRV_ATR_AclGroupSloppy,
+	SRV_ATR_KeepCompleted,
+	SRV_ATR_SubmitHosts,
+	SRV_ATR_AllowNodeSubmit,
+	SRV_ATR_AutoNodeNP,
+	SRV_ATR_LogFileMaxSize,
+	SRV_ATR_LogFileRollDepth,
 #include "site_svr_attr_enum.h"
+	SRV_ATR_ServerName,
 	/* This must be last */
 	SRV_ATR_LAST
 };
@@ -186,10 +198,11 @@ extern struct server server;
 #define SVR_HOT_CYCLE	15	/* retry mom every n sec on hot start     */
 #define SVR_HOT_LIMIT	300	/* after n seconds, drop out of hot start */
 
+
 /* function prototypes */
 
-extern int  svr_recov A_((char *));
-extern int  svr_save A_((struct server *, int mode));
+extern int svr_recov A_((char *));
+extern int svr_save A_((struct server *,int));
 
 
 /*
@@ -207,4 +220,11 @@ extern int  svr_save A_((struct server *, int mode));
 #define JobStatRate MAX(PBS_JOBSTAT_MIN,server.sv_attr[(int)SRV_ATR_JobStatRate].at_val.at_long)
 #endif
 
+
+/* maintain a list of new nodes */
+typedef struct new_node {
+        list_link            nn_link;   /* link to other resources in list */
+	char                *nn_name;   /* name of new node */
+} new_node;                                                                                           
+
 /* END server.h */
