@@ -93,6 +93,11 @@
 
 #include "dis.h"
 #include "dis_init.h"
+#include "log.h"
+
+#ifdef HAVE_SYS_POLL_H
+#include <sys/poll.h>
+#endif
 
 static struct tcp_chan	**tcparray = NULL;
 static int		  tcparraymax = 0;
@@ -185,8 +190,13 @@ static int tcp_read(
  
   {
   int               i;
+#ifdef HAVE_POLL
+  struct pollfd pollset;
+  int timeout;
+#else
   fd_set            readset;
   struct timeval    timeout;
+#endif
   struct tcpdisbuf *tp;
 
   tp = &tcparray[fd]->readbuf;
@@ -202,9 +212,23 @@ static int tcp_read(
    */
 
   tcparray[fd]->IsTimeout = 0;
+  tcparray[fd]->SelectErrno = 0;
+  tcparray[fd]->ReadErrno = 0;
 
   do 
     {
+#ifdef HAVE_POLL
+    /* poll()'s timeout is only a signed int, must be careful not to overflow */
+    if (INT_MAX/1000 > pbs_tcp_timeout)
+      timeout = pbs_tcp_timeout*1000;
+    else
+      timeout=INT_MAX;
+
+    pollset.fd = fd;
+    pollset.events = POLLIN|POLLHUP;
+
+    i = poll(&pollset,1,timeout);
+#else
     timeout.tv_sec = pbs_tcp_timeout;
     timeout.tv_usec = 0;
 
@@ -212,23 +236,31 @@ static int tcp_read(
     FD_SET(fd,&readset);
 
     i = select(
-      FD_SETSIZE, 
+      fd+1, 
       &readset, 
       NULL,
       NULL, 
       &timeout);
+#endif
     } while ((i == -1) && (errno == EINTR));
 
-  if (i <= 0)
+  if (i == 0)
     {
-    /* if i == 0, timeout has occurred */
+    /* timeout has occurred */
 
-    if (i == 0)
-      tcparray[fd]->IsTimeout = 1;
+    tcparray[fd]->IsTimeout = 1;
 
-    return(i);
+    return(0);
     }
-	
+  else if (i < 0)
+    {
+    /* select failed */
+
+    tcparray[fd]->SelectErrno = errno;
+
+    return(-1);
+    }
+  
   while ((i = read(
       fd,
       tp->tdis_eod,
@@ -238,10 +270,26 @@ static int tcp_read(
       break;
     }
 
-  if (i > 0)
-    tp->tdis_eod += i;
+  if (i < 0)
+    {
+    /* FAILURE - read failed */
 
-  return((i == 0) ? -2 : i);
+    tcparray[fd]->ReadErrno = errno;
+
+    return(-1);
+    }
+  else if (i == 0)
+    {
+    /* FAILURE - no data read */
+
+    return(-2);
+    }
+
+  /* SUCCESS */
+
+  tp->tdis_eod += i;
+ 
+  return(i);
   }  /* END tcp_read() */
 
 
@@ -334,7 +382,7 @@ void DIS_tcp_reset(
     DIS_tcp_clear(&tcp->writebuf);
 
   return;
-  }
+  }  /* END DIS_tcp_reset() */
 
 
 
@@ -464,6 +512,8 @@ int PConnTimeout(
 
   if (tcparray[sock]->IsTimeout == 1)
     {
+    /* timeout occurred, report 'TRUE' */
+
     return(1);
     }
 
@@ -473,6 +523,40 @@ int PConnTimeout(
 
 
 
+int TConnGetReadErrno(
+
+  int sock)  /* I */
+
+  {
+  if ((tcparray == NULL) || (tcparray[sock] == NULL))
+    {
+    return(0);
+    }
+
+  return(tcparray[sock]->ReadErrno);
+  }  /* END TConnGetReadErrno() */
+
+
+
+
+
+int TConnGetSelectErrno(
+
+  int sock)  /* I */
+
+  {
+  if ((tcparray == NULL) || (tcparray[sock] == NULL))
+    {
+    return(0);
+    }
+
+  return(tcparray[sock]->SelectErrno);
+  }  /* END TConnGetSelectErrno() */
+
+
+
+
+
 /*
  * tcp_puts - tcp/dis support routine to put a counted string of characters
  *	into the write buffer.
@@ -483,15 +567,21 @@ int PConnTimeout(
 
 static int tcp_puts(
 
-  int         fd,
-  const char *str,
-  size_t      ct)
+  int         fd,  /* I */
+  const char *str, /* I */
+  size_t      ct)  /* I */
 
   {
+  char *id = "tcp_puts";
+
   struct tcpdisbuf *tp;
 
   tp = &tcparray[fd]->writebuf;
 
+  /* NOTE:  currently, failures may occur if THE_BUF_SIZE is not large enough */
+  /*        this should be changed to allow proper operation with degraded    */
+  /*        performance (how?) */
+
   if ((tp->tdis_thebuf + THE_BUF_SIZE - tp->tdis_leadp) < (ssize_t)ct) 
     {
     /* not enough room, try to flush committed data */
@@ -499,7 +589,15 @@ static int tcp_puts(
     if ((DIS_tcp_wflush(fd) < 0) ||
        ((tp->tdis_thebuf + THE_BUF_SIZE - tp->tdis_leadp) < (ssize_t)ct))
       {
-      return(-1);		/* error */
+      /* FAILURE */
+
+      DBPRT(("%s: error!  out of space in buffer and cannot commit message (bufsize=%d, buflen=%d, ct=%d)\n",
+        id,
+        THE_BUF_SIZE,
+        (int)(tp->tdis_leadp - tp->tdis_thebuf),
+        (int)ct))
+
+      return(-1);	
       }
     }
 
@@ -508,7 +606,7 @@ static int tcp_puts(
   tp->tdis_leadp += ct;
 
   return(ct);
-  }
+  }  /* END tcp_puts() */
 
 
 
@@ -530,16 +628,18 @@ static int tcp_rcommit(
   if (commit_flag) 
     {
     /* commit by moving trailing up */
+
     tp->tdis_trailp = tp->tdis_leadp;
     } 
   else 
     {
     /* uncommit by moving leading back */
+
     tp->tdis_leadp = tp->tdis_trailp;
     }
 
   return(0);
-  }
+  }  /* END tcp_rcommit() */
 
 
 
