@@ -85,6 +85,8 @@
  *	log_err()
  *	log_record()
  *	log_close()
+ *	log_roll()
+ *	log_size()
  */
 
 #include <pbs_config.h>   /* the master config generated by configure */
@@ -93,6 +95,7 @@
 #include "pbs_error.h"
 
 #include <sys/param.h>
+#include <sys/stat.h>
 #include <sys/types.h>
 #include <limits.h>
 #include <time.h>
@@ -108,18 +111,19 @@
 #include <syslog.h>
 #endif
 
-/* Global Data */
 
+/* Global Data */
 char log_buffer[LOG_BUF_SIZE];
 char log_directory[_POSIX_PATH_MAX/2];
 
-extern char *msg_daemonname;
+char *msg_daemonname = "unset";
 
 /* Local Data */
 
 static int	     log_auto_switch = 0;
 static int	     log_open_day;
 static FILE	    *logfile;		/* open stream for log file */
+static char	    *logpath = NULL;
 static volatile int  log_opened = 0;
 #if SYSLOG
 static int	     syslogopen = 0;
@@ -223,6 +227,14 @@ int log_open(
     fds = log_opened;
     }
 
+  /* save the path of the last opened logfile for log_roll */
+  if (logpath != filename)
+     {
+     if (logpath != NULL)
+        free(logpath);
+     logpath = strdup(filename);
+     }
+
   logfile = fdopen(fds,"a");
 
   setvbuf(logfile,NULL,_IOLBF,0);	/* set line buffering */
@@ -450,4 +462,120 @@ void log_close(
   }  /* END log_close() */
 
 
+void log_roll(int max_depth)
+   {
+   int i, suffix_size, file_buf_len, as;
+   int err = 0;
+   char *source  = NULL; 
+   char *dest    = NULL;
 
+   if (!log_opened)
+      {
+      return;
+      }
+   
+   /* save value of log_auto_switch */
+   as = log_auto_switch;
+
+   log_close(1);
+
+   /* find out how many characters the suffix could be. (save in suffix_size)
+      start at 1 to account for the "."  */
+   for (i = max_depth, suffix_size=1; i > 0; suffix_size++, i /= 10)
+      ;
+
+   /* allocate memory for rolling */
+   file_buf_len = sizeof(char) * (strlen(logpath) + suffix_size + 1); 
+   source = (char*)malloc(file_buf_len);
+   dest   = (char*)malloc(file_buf_len);
+
+   /* call unlink to delete logname.max_depth - it doesn't matter if it 
+      doesn't exist, so we'll ignore ENOENT */
+
+   sprintf(dest, "%s.%d", logpath, max_depth);
+ 
+   if (unlink(dest) != 0 && errno != ENOENT)
+      {
+      err = errno;
+      goto done_roll;
+      }
+
+
+   /* logname.max_depth is gone, so roll the rest of the log files */
+   for (i = max_depth - 1; i >= 0; i--)
+     {
+
+     if (i == 0)
+        {
+        strcpy(source, logpath);
+        }
+     else
+        {
+        sprintf(source, "%s.%d", logpath, i);
+        }
+     sprintf(dest, "%s.%d", logpath, i+1);
+
+     /* rename file if it exists */
+     if (rename(source, dest) != 0 && errno != ENOENT)
+        {
+        err = errno;
+        goto done_roll;
+        }
+
+     }
+   
+
+done_roll:
+
+   if (as)
+      {
+      log_open(NULL, log_directory);
+      }
+   else
+      {
+      log_open(logpath, log_directory);   
+      }
+
+   free(source);
+   free(dest);
+
+   if (err != 0)
+      {
+      log_err(err, "log_roll", "error while rollng logs");
+      }
+   else
+     {
+      log_record(
+        PBSEVENT_SYSTEM,
+        PBS_EVENTCLASS_SERVER,
+        "Log",
+        "Log Rolled");
+     }
+  
+   } /* END log_roll() */
+
+
+
+/* return size of log file in kilobyes */
+long log_size(void)
+{
+#if defined(HAVE_STRUCT_STAT64) && defined(HAVE_STAT64)
+   struct stat64 file_stat;
+#else
+   struct stat file_stat;
+#endif
+
+#if defined(HAVE_STRUCT_STAT64) && defined(HAVE_STAT64)
+   if (log_opened && fstat64(fileno(logfile), &file_stat) == 0)
+#else
+   if (log_opened && fstat(fileno(logfile), &file_stat) == 0)
+#endif
+      {
+      return (file_stat.st_size / 1024);
+      }
+   else
+      {
+      log_err(errno,"log_size","PBS cannot fstat logfile");
+      return 0;
+      }
+}
