@@ -607,6 +607,8 @@ static unsigned long cput_sum(pjob)
 
 	if (nps == 0)
 		pjob->ji_flags |= MOM_NO_PROC;
+	else
+		pjob->ji_flags &= ~MOM_NO_PROC;
 
 	return ((unsigned long)((double)cputime * cputfactor));
 }
@@ -1363,9 +1365,10 @@ int mom_set_use(pjob)
  *	Kill a task session.
  *	Call with the job pointer and a signal number.
  */
-int kill_task(ptask, sig)
+int kill_task(ptask, sig,pg)
     task	*ptask;
     int  	sig;
+    int         pg;
 {
 	int	ct = 0;
 	int	i, err;
@@ -1905,7 +1908,10 @@ struct	rm_attribute	*attrib;
 	fmt = ret_string;
 	for (j=0; j<njids; j++) {
 		checkret(&fmt, 100);
-		sprintf(fmt, " %d", jids[j]);
+                if (j==0)
+                  sprintf(fmt, "%d", jids[j]);
+                else
+		  sprintf(fmt, " %d", jids[j]);
 		fmt += strlen(fmt);
 	}
 	free(jids);
@@ -1917,7 +1923,7 @@ nsessions(attrib)
 struct	rm_attribute	*attrib;
 {
 	char	*result, *ch;
-	int	num = 0;
+	int	num = 1;
 
 	if ((result = sessions(attrib)) == NULL)
 		return result;
