@@ -104,6 +104,7 @@
 #include	<stdio.h>
 #include	<stdbool.h>
 #include	<stdlib.h>
+#include	<unistd.h>
 #include	<ctype.h>
 #include	<dirent.h>
 #include	<string.h>
@@ -138,14 +139,14 @@
 #define TUSEMEMSIZE 1
 
 /*  additional header files required for darwin 8.0.0 */
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
 #include 	<mach/mach_host.h>
 #include	<mach/mach_port.h>
 #include	<mach/mach_traps.h>
 #include	<mach/shared_memory_server.h>
 #include	<mach/task.h>
 #include	<mach/thread_act.h>
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
 /* uncomment the following two header files if required for local build */
 /*
@@ -163,6 +164,7 @@
 #include	"job.h"
 #include	"mom_mach.h"
 #include	"resmon.h"
+#include	"mom_func.h"
 #include	"../rm_dep.h"
 
 #ifndef TRUE
@@ -198,11 +200,11 @@ static char *netload	A_((struct rm_attribute *attrib));
 extern char *loadave	A_((struct rm_attribute *attrib));
 extern char *nullproc	A_((struct rm_attribute *attrib));
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
 int		get_tinfo_by_pid  A_((struct task_basic_info *t_info, unsigned int pid));
-#else /* __TDARWIN_8 */
+#else /* TDARWIN8 */
 int		bs_cmp		A_((const void *key, const void *member));
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
 struct	config	dependent_config[] = {
   { "resi",	{resi} },
@@ -217,10 +219,10 @@ struct	config	dependent_config[] = {
   { NULL,	{nullproc} } };
 
 struct nlist nl[] = {
-	{ "_anoninfo" },	/* 0 */
-	{ "_cnt" },		/* 1 */
-	{ "_averunnable" },	/* 2 */
-	{ "" }
+	{ "_anoninfo",0,0,0,0 },	/* 0 */
+	{ "_cnt",0,0,0,0 },		/* 1 */
+	{ "_averunnable",0,0,0,0 },	/* 2 */
+	{ "",0,0,0,0 }
 };
 
 #define	KSYM_ANON		0
@@ -517,7 +519,7 @@ static unsigned long cput_sum(
       cputime += tv(ru.ru_utime) + tv(ru.ru_stime);
       }
 
-    DBPRT(("%s: ses %d pid %d cputime %d\n", 
+    DBPRT(("%s: ses %d pid %d cputime %lu\n", 
       id,
       sess_tbl[i], 
       pp->kp_proc.p_pid, 
@@ -526,6 +528,8 @@ static unsigned long cput_sum(
  
   if (nps == 0)
     pjob->ji_flags |= MOM_NO_PROC;
+  else
+    pjob->ji_flags &= ~MOM_NO_PROC;
 
   return((unsigned long)((double)cputime * cputfactor));
   } /* END cput_sum() */
@@ -551,11 +555,11 @@ static unsigned long mem_sum(
   int           i;
   unsigned long memsize = 0;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
   struct task_basic_info t_info;
 
   unsigned int pid;
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
   DBPRT(("mem_sum entered.\n"));
 
@@ -566,18 +570,18 @@ static unsigned long mem_sum(
     if (!injob(pjob,sess_tbl[i]))
       continue;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
     pid = pp->kp_proc.p_pid;
 
     if (get_tinfo_by_pid(&t_info,pid) != 0)
       continue;
 
     memsize += ctob(t_info.virtual_size);	
-#else /* __TDARWIN_8 */
+#else /* TDARWIN8 */
     memsize += ctob(pp->kp_eproc.e_vm.vm_tsize +
       pp->kp_eproc.e_vm.vm_dsize +
       pp->kp_eproc.e_vm.vm_ssize);
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
     DBPRT(("%s: ses %d pid=%d totmem=%lu\n", 
       id, sess_tbl[i], pp->kp_proc.p_pid, memsize))
@@ -602,10 +606,10 @@ static unsigned long resi_sum(
   int		 i;
   unsigned long	 memsize = 0;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
   struct task_basic_info t_info;	
   unsigned int pid;
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
   for (i = 0;i < nproc;i++) 
     {
@@ -614,7 +618,7 @@ static unsigned long resi_sum(
     if (!injob(pjob,sess_tbl[i]))
       continue;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
     pid = pp->kp_proc.p_pid;
 
     if (get_tinfo_by_pid(&t_info,pid) != 0)
@@ -624,12 +628,12 @@ static unsigned long resi_sum(
 
     DBPRT(("%s: pid=%d ses=%d mem=%d totmem=%d\n", 
      id,pp->kp_proc.p_pid,sess_tbl[i],t_info.resident_size,memsize))
-#else /* __TDARWIN_8 */
+#else /* TDARWIN8 */
     memsize += ctob(pp->kp_eproc.e_vm.vm_rssize);
 
-    DBPRT(("%s: pid=%d ses=%d mem=%d totmem=%d\n",
+    DBPRT(("%s: pid=%d ses=%d mem=%d totmem=%lu\n",
       id,pp->kp_proc.p_pid,sess_tbl[i],pp->kp_eproc.e_vm.vm_rssize,memsize))
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
     }  /* END for (i) */
 
   return(memsize);
@@ -651,10 +655,10 @@ static int overmem_proc(
   {
   int i;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
   struct task_basic_info t_info;	
   unsigned int pid;	
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
   for (i = 0;i < nproc;i++) 
     {
@@ -663,7 +667,7 @@ static int overmem_proc(
     if (!injob(pjob,sess_tbl[i]))
       continue;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
     pid = pp->kp_proc.p_pid;
 
     if (get_tinfo_by_pid(&t_info,pid) != 0)
@@ -673,14 +677,14 @@ static int overmem_proc(
       {
       return(TRUE);	
       }
-#else /* __TDARWIN_8 */
-    if (ctob(pp->kp_eproc.e_vm.vm_tsize +
+#else /* TDARWIN8 */
+    if ((unsigned long)ctob(pp->kp_eproc.e_vm.vm_tsize +
              pp->kp_eproc.e_vm.vm_dsize +
              pp->kp_eproc.e_vm.vm_ssize) > limit)
       {
       return(TRUE);
       }
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
     }  /* END for (i) */
 
   return(FALSE);
@@ -949,12 +953,12 @@ int qs_cmp(
 
 
 /* removed 5/22/05 MAINE OSX 10.5 patch 
- * Only needs to be removed if __TDARWIN_8
+ * Only needs to be removed if TDARWIN8
  * is defined
  */
 
 
-#ifndef __TDARWIN_8
+#ifndef TDARWIN8
 
 int bs_cmp(
 
@@ -965,7 +969,7 @@ int bs_cmp(
   return((int)((struct session *)key)->s_leader - (int)((struct kinfo_proc *)member)->kp_eproc.e_paddr);
   }
 
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
 typedef struct kinfo_proc kinfo_proc;
 
@@ -1089,7 +1093,7 @@ static kinfo_proc *GetBSDProcessList(
  *	use the same data.  Returns a PBS error code.
  */
 
-#ifndef __TDARWIN_8
+#ifndef TDARWIN8
 
 int mom_get_sample()
 
@@ -1183,7 +1187,7 @@ int mom_get_sample()
   return(PBSE_NONE);
   }
 
-#else /* __TDARWIN_8 */
+#else /* TDARWIN8 */
 
 int mom_get_sample()
 
@@ -1248,7 +1252,7 @@ int mom_get_sample()
   return(PBSE_NONE);
   }
 
-#endif /* !__TDARWIN_8 */
+#endif /* !TDARWIN8 */
 
 
 
@@ -1362,7 +1366,7 @@ int mom_over_limit(
 
       if (num > value) 
         {
-        sprintf(log_buffer,"walltime %d exceeded limit %lu",
+        sprintf(log_buffer,"walltime %lu exceeded limit %lu",
           num,value);
 
 	if (ignwalltime == 0)
@@ -1486,7 +1490,8 @@ int mom_set_use(pjob)
 int kill_task(
 
   task *ptask,
-  int   sig)
+  int   sig,
+  int   pg)
 
   {
   char	*id = "kill_task";
@@ -1540,7 +1545,7 @@ int kill_task(
       }
  
     sprintf(log_buffer,"%s: killing pid %d task %d with sig %d",
-      (char *)__func__,
+      id,
       pp->kp_proc.p_pid,
       ptask->ti_qs.ti_task,
       sig);
@@ -1713,7 +1718,7 @@ pid_t	jobid;
 			}
 			cputime += tv(ru.ru_utime) + tv(ru.ru_stime);
 		}
-		DBPRT(("%s: ses %d pid %d cputime %d\n", id,
+		DBPRT(("%s: ses %d pid %d cputime %lu\n", id,
 				jobid, pp->kp_proc.p_pid, cputime))
 
 	}
@@ -1733,7 +1738,8 @@ char *cput_proc(
   {
   char		*id = "cput_proc";
   struct	pstats		ps;
-  uint		i, cputime;
+  uint		cputime;
+  int           i;
 
   if (kd == NULL)
     {
@@ -1857,10 +1863,10 @@ char *mem_job(
   int   memsize, addmem;
   int   found = 0;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
   struct task_basic_info t_info;
   unsigned int pid;
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
   if (getprocs() == 0) 
     {
@@ -1880,22 +1886,22 @@ char *mem_job(
 
     found = 1;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
     pid = pp->kp_proc.p_pid;
 
     if (get_tinfo_by_pid(&t_info,pid) != 0)
       continue;
 
     addmem = t_info.virtual_size;
-#else /* __TDARWIN_8 */
+#else /* TDARWIN8 */
     addmem = pp->kp_eproc.e_vm.vm_tsize +
              pp->kp_eproc.e_vm.vm_dsize +
              pp->kp_eproc.e_vm.vm_ssize;
-#endif /* !__TDARWIN_8 */
+#endif /* !TDARWIN8 */
 
     memsize += addmem;
 
-    DBPRT(("%s: ses %d pid=%d totmem=%lu\n", 
+    DBPRT(("%s: ses %d pid=%d totmem=%d\n", 
       id, sess_tbl[i], pp->kp_proc.p_pid, memsize));
     }
 
@@ -1924,9 +1930,9 @@ char *mem_proc(
   int   i;
   int   memsize;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
   struct task_basic_info t_info;
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
   if (getprocs() == 0)
     {
@@ -1944,18 +1950,18 @@ char *mem_proc(
     if (pid != sess_tbl[i])
       continue;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
     pid = pp->kp_proc.p_pid;
 
     if (get_tinfo_by_pid(&t_info,pid) != 0)
       continue;
 
     memsize = t_info.virtual_size;
-#else /* __TDARWIN_8 */
+#else /* TDARWIN8 */
     memsize = pp->kp_eproc.e_vm.vm_tsize +
               pp->kp_eproc.e_vm.vm_dsize +
               pp->kp_eproc.e_vm.vm_ssize;
-#endif /* !__TDARWIN_8 */
+#endif /* !TDARWIN8 */
 
     sprintf(ret_string,"%ukb", 
       ctob(memsize) >> 10); /* KB */
@@ -2018,10 +2024,10 @@ static char *resi_job(
   int	i, found;
   int	resisize;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
   struct task_basic_info t_info;
   unsigned int pid;
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
   if (getprocs() == 0) 
     {
@@ -2042,16 +2048,16 @@ static char *resi_job(
 
     found = 1;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
     pid = pp->kp_proc.p_pid;
 
     if (get_tinfo_by_pid(&t_info,pid ) != 0)
       continue;
 
     resisize += t_info.resident_size;
-#else /* __TDARWIN_8 */
+#else /* TDARWIN8 */
     resisize += pp->kp_eproc.e_vm.vm_rssize;
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
     }
 
   if (found) 
@@ -2079,9 +2085,9 @@ static char *resi_proc(
   int	i;
   int	resisize;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
   struct task_basic_info t_info;
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
   if (getprocs() == 0) 
     {
@@ -2099,14 +2105,14 @@ static char *resi_proc(
     if (pid != pp->kp_proc.p_pid)
       continue;
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
     if (get_tinfo_by_pid(&t_info,pp->kp_proc.p_pid) != 0)
       continue;
 
     resisize = t_info.resident_size;
-#else /* __TDARWIN_8 */
+#else /* TDARWIN8 */
     resisize = pp->kp_eproc.e_vm.vm_rssize;
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
     sprintf(ret_string,"%ukb",
       ctob(resisize) >> 10); /* KB */
@@ -2263,7 +2269,7 @@ char *nsessions(
 
   {
   char *result, *ch;
-  int   num = 0;
+  int   num = 1;
 
   if ((result = sessions(attrib)) == NULL)
     { 
@@ -2792,7 +2798,7 @@ char *size_file(
 		return NULL;
 	}
 
-	sprintf(ret_string, "%ukb", sbuf.st_size >> 10); /* in KB */
+	sprintf(ret_string, "%lldkb", sbuf.st_size >> 10); /* in KB */
 	return ret_string;
 }
 
@@ -2843,8 +2849,8 @@ void setmax(
 
   if (maxtm < sb.st_atime) 
     {
-    DBPRT(("setmax: dev %s access %d replaces max %ld\n",
-      dev,sb.st_atime,(long)maxtm))
+    DBPRT(("setmax: dev %s access %ld replaces max %ld\n",
+      dev,(long)sb.st_atime,(long)maxtm))
 
     maxtm = sb.st_atime;
     }
@@ -3039,7 +3045,7 @@ u_long	secs;
 {
 	time_t	now = time((time_t *)NULL);
 
-	if (secs > now)		/* time is in the future */
+	if (secs > (u_long)now)		/* time is in the future */
 		return (secs - now);
 	else
 		return 0;
@@ -3209,10 +3215,10 @@ struct	rm_attribute	*attrib;
 		sprintf(ret_string, "%u", qi.dqb_curinodes);
 		break;
 	case timedata:
-		sprintf(ret_string, "%u", gracetime(qi.dqb_btime));
+		sprintf(ret_string, "%lu", gracetime(qi.dqb_btime));
 		break;
 	case timefile:
-		sprintf(ret_string, "%u", gracetime(qi.dqb_itime));
+		sprintf(ret_string, "%lu", gracetime(qi.dqb_itime));
 		break;
 	}
 
@@ -3233,7 +3239,7 @@ void scan_non_child_tasks(void)
 
 
 
-#ifdef __TDARWIN_8
+#ifdef TDARWIN8
 
 /* This is basically a wrapper for the MACH t_info and vm_region calls */
 
@@ -3302,5 +3308,5 @@ int get_tinfo_by_pid(
   return(0);
   }
 
-#endif /* __TDARWIN_8 */
+#endif /* TDARWIN8 */
 
