@@ -135,7 +135,7 @@
 **		quota		quota information (sizes in KB)
 */
 
-static char ident[] = "@(#) sgi/$RCSfile: mom_mach.c,v $ $Revision: 1.9 $";
+static char ident[] = "@(#) sgi/$RCSfile$ $Revision: 991 $";
 
 #ifndef TRUE
 #define FALSE	0
@@ -336,7 +336,7 @@ static int gettime(pres, ret)
 static
 int
 injob(phead, psp)
-    list_head	*phead;
+    tlist_head	*phead;
     prpsinfo_t	*psp;
 {
 	task		*ptask;
@@ -363,7 +363,7 @@ injob(phead, psp)
  */
 
 static unsigned long cput_sum(phead)
-    list_head	*phead;
+    tlist_head	*phead;
 {
 	char			*id = "cput_sum";
 	int			fd;
@@ -371,6 +371,7 @@ static unsigned long cput_sum(phead)
 	char			procname[100];
 	ulong			cputime, addtime;
 	prpsinfo_t		pi;
+	int			nps=0;
 
 	DBPRT(("%s: entered\n", id))
 	cputime = 0.0;
@@ -390,12 +391,18 @@ static unsigned long cput_sum(phead)
 		}
 		if (!injob(phead, &pi))
 			continue;
+		nps++;
 		addtime = tv(pi.pr_time) + tv(pi.pr_ctime);
 		cputime += addtime;
 		DBPRT(("%s: ses %d pid %d cputime %d\n",
 				id, pi.pr_sid, pi.pr_pid, addtime, cputime))
 	}
 
+	if (nps == 0)
+		pjob->ji_flags |= MOM_NO_PROC;
+	else    
+		pjob->ji_flags &= ~MOM_NO_PROC;
+
 	return ((unsigned long)((double)cputime * cputfactor));
 }
 
@@ -403,7 +410,7 @@ static unsigned long cput_sum(phead)
  * Return TRUE if any process in the job is over limit for cputime usage.
  */
 static int overcpu_proc(phead, limit)
-    list_head		*phead;
+    tlist_head		*phead;
     unsigned long	limit;
 {
 	char		*id = "overcpu_proc";
@@ -443,7 +450,7 @@ static int overcpu_proc(phead, limit)
  *	space consumed by all current tasks within the list of tasks.
  */
 static unsigned long mem_sum(phead)
-    list_head	*phead;
+    tlist_head	*phead;
 {
 	static	char		id[] = "mem_sum";
 	int			fd;
@@ -491,7 +498,7 @@ static unsigned long mem_sum(phead)
  * Internal session mem (workingset) size function.
  */
 static unsigned long resi_sum(phead)
-    list_head		*phead;
+    tlist_head		*phead;
 {
 	static	char		id[]="resi_ses";
 	ulong			resisize, resisub;
@@ -568,7 +575,7 @@ static unsigned long resi_sum(phead)
  * Return TRUE if any process in the session is over limit for memory usage.
  */
 static int overmem_proc(phead, limit)
-    list_head		*phead;
+    tlist_head		*phead;
     unsigned long	limit;
 {
 	char		*id = "overmem_proc";
@@ -1007,9 +1014,10 @@ int mom_set_use(pjob)
  *	Kill a task session.
  *	Call with the task pointer and a signal number.
  */
-int kill_task(ptask, sig)
+int kill_task(ptask, sig, pg)
     task	*ptask;
     int		sig;
+    int         pg;
 {
 	char		*id = "kill_task";
 	int		ct = 0;
@@ -1230,7 +1238,7 @@ mem_job(sid)
 pid_t	sid;
 {
 	ulong	memsize;
-	static	list_head	taskhead;
+	static	tlist_head	taskhead;
 	static	task		onetask;
 	static	int		first = 1;
 
@@ -1492,7 +1500,10 @@ struct	rm_attribute	*attrib;
 	fmt = ret_string;
 	for (j=0; j<njids; j++) {
 		checkret(&fmt, 100);
-		sprintf(fmt, " %d", (int)jids[j]);
+                if (j==0)
+		  sprintf(fmt, "%d", (int)jids[j]);
+                else
+		  sprintf(fmt, " %d", (int)jids[j]);
 		fmt += strlen(fmt);
 	}
 	free(jids);
@@ -1504,7 +1515,7 @@ nsessions(attrib)
 struct	rm_attribute	*attrib;
 {
 	char	*result, *ch;
-	int	num = 0;
+	int	num = 1;
 
 	if ((result = sessions(attrib)) == NULL)
 		return result;
