@@ -138,7 +138,7 @@ extern int TTmpDirName( job*,char *);
 
 extern unsigned int	default_server_port;
 extern int		exiting_tasks;
-extern list_head	svr_alljobs;
+extern tlist_head	svr_alljobs;
 extern char		mom_host[];
 extern char		*msg_err_unlink;
 extern char		*path_checkpoint;
@@ -152,6 +152,9 @@ extern int		resc_access_perm;	/* see encode_resc() */
 						/* in attr_fn_resc.c */
 
 extern char             MOMUNameMissing[];
+extern int              pbs_rm_port;
+extern char             rcp_path[];
+extern char             rcp_args[];
 
 /* Local Data Items */
 
@@ -165,18 +168,19 @@ static char   rcperr[MAXPATHLEN];	/* file to contain rcp error */
 extern char PBSNodeMsgBuf[1024];
 extern int  LOGLEVEL;
 
+extern int im_compose A_((int,char *,char *,int,tm_event_t,tm_task_id));
 
 
 /* prototypes */
 
-char *__get_variable(job *,char *);
+char *get_job_envvar(job *,char *);
 
 /* loaded in mom_mach.h */
 
 /* END prototypes */
 
 
-char *__get_variable(
+char *get_job_envvar(
 
   job  *pjob,     /* I */
   char *variable) /* I */
@@ -200,7 +204,7 @@ char *__get_variable(
     }
 
   return(pc);
-  }  /* END __get_variable() */
+  }  /* END get_job_envvar() */
 
 
 
@@ -250,7 +254,7 @@ static pid_t fork_to_user(
     ngroup  = pjob->ji_grpcache->gc_ngroup;
     groups  = pjob->ji_grpcache->gc_groups;
 
-    if ((idir = __get_variable(pjob,"PBS_O_INITDIR")) != NULL)
+    if ((idir = get_job_envvar(pjob,"PBS_O_INITDIR")) != NULL)
       {
       hdir = idir;
       }
@@ -289,7 +293,8 @@ static pid_t fork_to_user(
       }
     else
       {
-      sprintf(log_buffer,"cannot find group for user '%s' in password file",
+      sprintf(log_buffer,"cannot find group '%s' for user '%s' in password file",
+        preq->rq_ind.rq_cpyfile.rq_group,
         preq->rq_ind.rq_cpyfile.rq_user);
 
       if (EMsg != NULL)
@@ -309,7 +314,7 @@ static pid_t fork_to_user(
 
     /* determine user`s home directory */
 
-    if ((pjob != NULL) && ((idir = __get_variable(pjob,"PBS_O_INITDIR")) != NULL))
+    if ((pjob != NULL) && ((idir = get_job_envvar(pjob,"PBS_O_INITDIR")) != NULL))
       {
       hdir = idir;
       }
@@ -378,8 +383,8 @@ static pid_t fork_to_user(
       hdir);
  
     log_record(
-      PBSEVENT_ADMIN,
-      PBS_EVENTCLASS_FILE,
+      PBSEVENT_JOB,
+      PBS_EVENTCLASS_JOB,
       (pjob != NULL) ? pjob->ji_qs.ji_jobid : "N/A",
       log_buffer);
     }
@@ -557,6 +562,8 @@ static void add_bad_list(
 
 #define RT_BLK_SZ 4096 
 
+/* return 0 on failure */
+
 static int return_file(
 
   job	        *pjob,
@@ -620,7 +627,7 @@ static int return_file(
 
       break;
       }
-    }
+    }    /* END while ((amt = read()) */
 
   free_br(prq);
 
@@ -823,9 +830,17 @@ static int is_file_same(
   char *file2)
 
   {
+#if defined(HAVE_STRUCT_STAT64) && defined(HAVE_STAT64)
+  struct stat64 sb1, sb2;
+#else
   struct stat sb1, sb2;
+#endif
 
+#if defined(HAVE_STRUCT_STAT64) && defined(HAVE_STAT64)
+  if ((stat64(file1,&sb1) == 0) && (stat64(file2,&sb2) == 0)) 
+#else
   if ((stat(file1,&sb1) == 0) && (stat(file2,&sb2) == 0)) 
+#endif
     {
     if (!memcmp(&sb1.st_dev,&sb2.st_dev,sizeof(dev_t)) && 
         !memcmp(&sb1.st_ino,&sb2.st_ino,sizeof(ino_t)) )
@@ -866,7 +881,7 @@ void req_deletejob(
     }
 
   return;
-  }
+  }  /* END req_deletejob() */
 
 
 
@@ -1204,13 +1219,13 @@ void req_modifyjob(
             newattr[i].at_val.at_long);
           }         
 
-        sprintf(log_buffer,"modifying attribute %s of job (value: '%s'",
+        sprintf(log_buffer,"modifying attribute %s of job (value: '%s')",
           TJobAttr[i],
           tmpLine);
 
         log_record(
-          PBSEVENT_ADMIN,
-          PBS_EVENTCLASS_FILE,
+          PBSEVENT_JOB,
+          PBS_EVENTCLASS_JOB,
           (pjob != NULL) ? pjob->ji_qs.ji_jobid : "N/A",
           log_buffer);
         }
@@ -1273,10 +1288,10 @@ void req_modifyjob(
 
 void req_shutdown(
 
-  struct batch_request *preq)
+  struct batch_request *preq)  /* I */
 
   {
-  req_reject(PBSE_NOSUP, 0, preq,NULL,NULL);
+  req_reject(PBSE_NOSUP,0,preq,NULL,NULL);
 
   return;
   }
@@ -1374,12 +1389,14 @@ static void cray_susp_resum(
   reply_ack(preq);
 
   exit(0);
-  }
+  }  /* END cray_susp_resum() */
 
 #endif	/* _CRAY */
 
 
 
+
+
 int MUSleep(
 
   long SleepDuration) /* I (in us) */
@@ -1397,152 +1414,271 @@ int MUSleep(
 
 
 
+/* send a signal to all tasks on sisters */
+int sigalltasks_sisters(job *pjob, int signum)
+  {
+  char      id[] = "sigalltasks_sisters";
+  char     *cookie;
+  eventent *ep;
+  int i;
+
+  cookie = pjob->ji_wattr[(int)JOB_ATR_Cookie].at_val.at_str;
+
+  for (i = 0;i < pjob->ji_numnodes;i++)
+    {
+    int ret;
+    hnodent *np = &pjob->ji_hosts[i];
+
+    if (np->hn_node == pjob->ji_nodeid) /* this is me */
+      continue;
+
+    DBPRT(("%s: sending sig%d to all tasks on sister %s\n",id,signum,np->hn_host));
+
+    if (np->hn_stream == -1)
+      np->hn_stream = rpp_open(np->hn_host,pbs_rm_port,NULL);
+
+    ep = event_alloc(IM_SIGNAL_TASK,np,TM_NULL_EVENT,TM_NULL_TASK);
+
+    if (np->hn_stream == -1)
+      {
+      np->hn_stream = rpp_open(np->hn_host,pbs_rm_port,NULL);
+      }
+
+    ret = im_compose(np->hn_stream,pjob->ji_qs.ji_jobid,cookie,IM_SIGNAL_TASK,ep->ee_event,TM_NULL_TASK);
+
+    if (ret != DIS_SUCCESS)
+      {
+      return ret;
+      }
+
+    ret = diswui(np->hn_stream, pjob->ji_nodeid); /* XXX */
+
+    if (ret != DIS_SUCCESS)
+      {
+      return ret;
+      }
+
+    ret = diswsi(np->hn_stream, TM_NULL_TASK);
+
+    if (ret != DIS_SUCCESS)
+      {
+      return ret;
+      }
+
+    ret = diswsi(np->hn_stream, signum);
+
+    if (ret != DIS_SUCCESS)
+      {
+      return ret;
+      }
+
+    ret = rpp_flush(np->hn_stream);
+
+    if (ret != DIS_SUCCESS)
+      {
+      return ret;
+      }
+    }
+
+    return 0;
+  }
+
+
 static void resume_suspend( 
 
   job                  *pjob,
-  int                   susp,  /* I */
+  int                   susp,  /* I (0=FALSE, 1=TRUE) */
   struct batch_request *preq)
 
   {
+  static char *id = "resume_suspend";
+
   task *tp;
 
   int   stat = 0;
   int   savederr = 0;
 
-  /* First we need to send SIGTSTP to let reasonable mpi daemons stop
-     their subprocesses, then we send SIGSTOP to stop all other types
-     of processes.  This is split into two loops instead of doing them
-     right after each other because one needs a delay between the
-     SIGTSTP and SIGSTOP to give the mpi daemon time to stop all its
-     tasks.  Inserting a delay for each task would produce a very long
-     delay for jobs with large number of tasks.
+  int   signum;
 
-     So, to summarize, we send all tasks SIGTSTP, then we wait for 1
-     second, then we send all tasks SIGSTOP.  Hopefully this should
-     cover all cases.  The delay should rather be configurable but I
-     don't have a clue how, nor time, to do that.
- 
-     Fortunately only one SIGCONT is needed to resume.
+  signum = (susp == 1) ? SIGSTOP : SIGCONT;
+   
+
+  if (LOGLEVEL >= 2)
+    {
+    sprintf(log_buffer,"%s: %s job",
+      id,
+      (susp == 1) ? "suspending" : "resuming");
+
+    log_record(
+      PBSEVENT_JOB,
+      PBS_EVENTCLASS_JOB,
+      (pjob != NULL) ? pjob->ji_qs.ji_jobid : "N/A",
+      log_buffer);
+    }
+
+  /* Once upon a time, suspend/resume signals weren't propagated to
+     sisters.  Verily, the parallel jobs weren't correctly suspended
+     or resumed.
+
+     Then it was decided that SIGTSTP would be used instead of SIGSTOP
+     so that specially patched MPI launchers could suspend the sisters.
+     To keep serial jobs suspending, a delayed SIGSTOP was added after
+     the SIGTSTP.
+
+     Of course, non-MPI parallel jobs were still not suspended properly.
+
+     Now we are sending SIGTSTP only to the top-level task on MS, then
+     SIGSTOP to all tasks and given MOM the ability to propagate
+     suspend/resume signals to sisters.
+
+                             The End.
    */
 
-  for (tp = (task *)GET_NEXT(pjob->ji_tasks);tp != NULL;tp = (task *)GET_NEXT(tp->ti_jobtask)) 
+
+  /* NOTE:  format {suspend[:X]|resume[:X]} should be supported to allow 
+            job state change AND custom suspend/resume signal (NYI) */
+
+  if (susp == 1)
     {
-    if (susp != 0)
-      {
-      /* NOTE:  for suspend, changed signals to SIGTSTP for MPI jobs - NORWAY */
+    kill_task((task *)GET_NEXT(pjob->ji_tasks),SIGTSTP,0);
 
-      /* stat = kill_task(tp,SIGSTOP); */
+    MUSleep(50000);
+    }
+    
+  for (tp = (task *)GET_NEXT(pjob->ji_tasks);
+       tp != NULL;
+       tp = (task *)GET_NEXT(tp->ti_jobtask)) 
+    {
+    if (tp->ti_qs.ti_status != TI_STATE_RUNNING)
+      continue;
 
-      stat = kill_task(tp,SIGTSTP);
-      }
-    else
-      {
-      stat = kill_task(tp,SIGCONT);
-      }
+    DBPRT(("%s: inspecting %d from node %d\n",
+      id,
+      tp->ti_qs.ti_task,
+      tp->ti_qs.ti_parentnode));
+
+    stat = kill_task(tp,signum,0);
 
     if (stat < 0) 
       {
       /* couldn't send signal, don't signal more tasks */
 
       savederr = errno;
-
       break;
-      }
+      }  /* END if (stat < 0) */
     }    /* END for (tp) */
 
-  if ((susp != 0) && (stat >= 0))
+  if (stat >= 0)
     {
-    MUSleep(50000);
- 
-    for (tp = (task *)GET_NEXT(pjob->ji_tasks);tp != NULL;tp = (task *)GET_NEXT(tp->ti_jobtask)) 
+    if (pjob->ji_numnodes > 1)
       {
-      stat = kill_task(tp,SIGSTOP);
+      stat = sigalltasks_sisters(pjob,signum);
 
-      if (stat < 0) 
+      if (stat < 0)
         {
-        /* couldn't send signal, don't signal more tasks */
-
         savederr = errno;
-
-        break;
         }
-      }    /* END for (tp) */
-    }      /* END if  ((susp != 0) && (stat >= 0)) */
+      }
+    }
 
-  /* We're done sending signals, let's adjust some statuses */
-
-  if (stat < 0) 
+  if (stat < 0)
     {
     /* We couldn't signal all the tasks, signal them back to their old state */
-    
-    for (tp = (task *)GET_NEXT(pjob->ji_tasks);tp != NULL;tp = (task *)GET_NEXT(tp->ti_jobtask)) 
+
+    if (LOGLEVEL >= 1)
       {
-      if (susp)
-        {
-        stat = kill_task(tp,SIGCONT);
-        }
-      else
-        {
-        /* NOTE:  for suspend, changed signals to SIGTSTP for MPI jobs - NORWAY */
-
-	/* stat = kill_task(tp,SIGSTOP); */
-
-        stat = kill_task(tp,SIGTSTP);
-        }
-      }  /* END for (tp) */
-
-    /* the same two signal exercise as above */
-
-    if (susp != 0)
-      {
-      MUSleep(50000);
-
-      for (tp = (task *)GET_NEXT(pjob->ji_tasks);tp != NULL;tp = (task *)GET_NEXT(tp->ti_jobtask)) 
-        {
-        stat = kill_task(tp,SIGSTOP);
-        }  /* END for (tp) */
+      sprintf(log_buffer,"cannot send signal %s to tasks of job in %s (errno=%d) - attempt aborted",
+        (susp == 1) ? "SIGSTOP" : "SIGCONT",
+        id,
+        savederr);
+      
+      log_record(
+        PBSEVENT_ERROR,
+        PBS_EVENTCLASS_JOB,
+        (pjob != NULL) ? pjob->ji_qs.ji_jobid : "N/A",
+        log_buffer);
       }
 
-    /* suspend/resume failued - report failure */
+    signum = (susp == 1) ? SIGCONT : SIGSTOP;
 
-    req_reject(PBSE_SYSTEM,savederr,preq,NULL,NULL);      
-    }  /* END if (stat < 0) */ 
+    for (tp = (task *)GET_NEXT(pjob->ji_tasks);                                                       
+         tp != NULL;                                                                                  
+         tp = (task *)GET_NEXT(tp->ti_jobtask))                                                       
+      {
+      if (tp->ti_qs.ti_status != TI_STATE_RUNNING)
+        continue;
+
+      kill_task(tp,signum,0);
+      }
+
+    if (pjob->ji_numnodes > 1)
+      {
+      sigalltasks_sisters(pjob,signum);
+      }
+
+    /* report suspend/resume failure */
+
+    req_reject(PBSE_SYSTEM,savederr,preq,NULL,NULL);
+
+    return;
+    }  /* END if (stat < 0) */
+
+
+  /* signals sent to all tasks, now adjust job state */
+
+  if (susp == 1) 
+    {
+    /* Successfully suspended, let's update status */
+    /* This is needed for calculating correct walltime */
+
+    pjob->ji_momstat = time_now;
+      
+    pjob->ji_qs.ji_substate = JOB_SUBSTATE_SUSPEND;
+    pjob->ji_qs.ji_svrflags |= JOB_SVFLG_Suspend;
+
+    if (LOGLEVEL >= 1)
+      {
+      log_record(
+        PBSEVENT_JOB,
+        PBS_EVENTCLASS_JOB,
+        (pjob != NULL) ? pjob->ji_qs.ji_jobid : "N/A",
+        "job suspended - adjusted job state");
+      }
+    } 
   else 
     {
-    if (susp != 0) 
-      {
-      /* Successfully suspended, let's update status */
-      /* This is needed for calculating correct walltime */
+    pjob->ji_qs.ji_substate = JOB_SUBSTATE_RUNNING;
 
-      pjob->ji_momstat = time_now;
+    /* Ok, we resumed'em, we have set ji_momstat to the time we suspended the
+       job.  We use this to compute a new start-time for the job, so that
+       walltime is computed correctly elsewhere */
       
-      pjob->ji_qs.ji_substate = JOB_SUBSTATE_SUSPEND;
-      pjob->ji_qs.ji_svrflags |= JOB_SVFLG_Suspend;
-      } 
-    else 
+    if (pjob->ji_qs.ji_svrflags & JOB_SVFLG_Suspend) 
       {
-      pjob->ji_qs.ji_substate = JOB_SUBSTATE_RUNNING;
+      /* If it's suspended, update the start time */
 
-      /* Ok, we resumed'em, we have set ji_momstat to the time we suspended the
-	 job.  We use this to compute a new start-time for the job, so that
-	 walltime is computed correctly elsewhere */
-      
-      if (pjob->ji_qs.ji_svrflags & JOB_SVFLG_Suspend) 
-        {
-        /* If it's suspended, update the start time */
-
-        pjob->ji_qs.ji_stime = pjob->ji_qs.ji_stime - pjob->ji_momstat + time_now;
-        }
-
-      /* Clear the Suspend bit */
-
-      pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_Suspend;
+      pjob->ji_qs.ji_stime = pjob->ji_qs.ji_stime - pjob->ji_momstat + time_now;
       }
 
-    /* Tell them we did it */
+    /* clear the Suspend bit */
 
-    reply_ack(preq);
-    }  /* else (stat < 0) */
+    pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_Suspend;
+
+    if (LOGLEVEL >= 1)
+      {
+      log_record(
+        PBSEVENT_JOB,
+        PBS_EVENTCLASS_JOB,
+        (pjob != NULL) ? pjob->ji_qs.ji_jobid : "N/A",
+        "job resumed - adjusted job state");
+      }
+    }    /* END else (susp != 0) */
+
+  /* acknowledge change to pbs_server */
+
+  reply_ack(preq);
+
+  /* SUCCESS */
  
   return;
   }  /* END resume_suspend() */
@@ -1559,7 +1695,7 @@ static void resume_suspend(
 
 void req_signaljob(
 
-  struct batch_request *preq)
+  struct batch_request *preq) /* I */
 
   {
   job            *pjob;
@@ -1579,10 +1715,19 @@ void req_signaljob(
 
   sname = preq->rq_ind.rq_signal.rq_signame;
 
-  DBPRT(("req_signaljob: Sending %s\n", 
-    sname))
+  if (LOGLEVEL >= 3)
+    {
+    sprintf(log_buffer,"signalling job with signal %s",
+      sname); 
 
-  if (strcmp(sname,SIG_SUSPEND) == 0) 
+    log_record(
+      PBSEVENT_JOB,
+      PBS_EVENTCLASS_JOB,
+      (pjob != NULL) ? pjob->ji_qs.ji_jobid : "N/A",
+      log_buffer);
+    }
+
+  if (!strcasecmp(sname,SIG_SUSPEND)) 
     {
     if (pjob->ji_qs.ji_substate != JOB_SUBSTATE_RUNNING) 
       {
@@ -1599,7 +1744,8 @@ void req_signaljob(
 
     return;
     } 
-  else if (strcmp(sname,SIG_RESUME) == 0) 
+
+  if (!strcasecmp(sname,SIG_RESUME)) 
     {
     if (pjob->ji_qs.ji_substate != JOB_SUBSTATE_SUSPEND) 
       {
@@ -1619,6 +1765,8 @@ void req_signaljob(
     return;
     }
 
+  /* standard signal received - signal tasks but do not change job state */
+
   if (isdigit((int)*sname))
     {
     sig = atoi(sname);
@@ -1653,7 +1801,7 @@ void req_signaljob(
     /* SIGNUL and no procs found, force job to exiting */
     /* force issue of (another) job obit */
 
-    sprintf(log_buffer,"Job recycled into exiting on SIGNULL from substate %d",
+    sprintf(log_buffer,"job recycled into exiting on SIGNULL from substate %d",
       pjob->ji_qs.ji_substate);
 
     LOG_EVENT(
@@ -1664,6 +1812,8 @@ void req_signaljob(
 
     pjob->ji_qs.ji_substate = JOB_SUBSTATE_EXITING;
 
+    job_save(pjob,SAVEJOB_QUICK);
+
     exiting_tasks = 1;
     }
 
@@ -1671,8 +1821,7 @@ void req_signaljob(
     {
     /* force issue of (another) job obit */
 
-    sprintf(log_buffer,"Job recycled into exiting on SIGKILL from substate %d",
-      pjob->ji_qs.ji_substate);
+    sprintf(log_buffer,"job recycled into exiting on SIGKILL from substate exiting");
 
     LOG_EVENT(
       PBSEVENT_ERROR, 
@@ -1680,8 +1829,6 @@ void req_signaljob(
       pjob->ji_qs.ji_jobid, 
       log_buffer);
 
-    pjob->ji_qs.ji_substate = JOB_SUBSTATE_EXITING;
-
     exiting_tasks = 1;
     }
 
@@ -1695,7 +1842,7 @@ void req_signaljob(
 void encode_used(
 
   job       *pjob,   /* I */
-  list_head *phead)  /* O */
+  tlist_head *phead)  /* O */
 
   {
   unsigned long		lnum;
@@ -1893,7 +2040,7 @@ void req_stat_job(
   reply_send(preq);
 
   return;
-  }  /* END req_statjob() */
+  }  /* END req_stat_job() */
 
 
 
@@ -2032,7 +2179,7 @@ static int del_files(
       if (S_ISDIR(sb.st_mode)) 
         {
         /* have a directory, must append last segment */
-        /* of source name to it for  the unlink	      */
+        /* of source name to it for the unlink	      */
 
 #if NO_SPOOL_OUTPUT == 1
         /* check for  ~/.pbs_spool */
@@ -2135,6 +2282,8 @@ void req_rerunjob(
   struct batch_request *preq)
 
   {
+  static char   *id = "req_rerunjob";
+
   job		*pjob;
   unsigned int	 port;
   int		 rc;
@@ -2179,27 +2328,40 @@ void req_rerunjob(
 
   if (sock < 0) 
     {
+    /* FAILURE */
+
     LOG_EVENT(
       PBSEVENT_ERROR,
       PBS_EVENTCLASS_REQUEST,
-      "req_rerun", 
+      id, 
       "no contact with the server");
 
-    req_reject(PBSE_NOSERVER, 0, preq,NULL,NULL);
+    req_reject(PBSE_NOSERVER,0,preq,NULL,NULL);
 
     exit(0);
     }
 
   if (((rc = return_file(pjob,StdOut,sock)) != 0) ||
-      ((rc = return_file(pjob, StdErr, sock)) != 0) ||
-      ((rc = return_file(pjob, Chkpt, sock)) != 0)) 
+      ((rc = return_file(pjob,StdErr,sock)) != 0) ||
+      ((rc = return_file(pjob,Chkpt,sock)) != 0)) 
     {
-    req_reject(rc, 0, preq,NULL,NULL);
+    /* FAILURE - cannot report file to server */
+
+    LOG_EVENT(
+      PBSEVENT_ERROR,
+      PBS_EVENTCLASS_REQUEST,
+      id,
+      "cannot move output files to server");
+
+    req_reject(rc,0,preq,NULL,NULL);
     }
   else
     {
+    /* SUCCESS */
+
     reply_ack(preq);
-    (void)close(sock);
+
+    close(sock);
 
     exit(0);
     }
@@ -2242,19 +2404,12 @@ static int sys_copy(
     /* local copy */
 
     ag0 = "/bin/cp";
-    ag1 = "-r";
+    ag1 = "-rp";
     } 
   else
     {
-    /* use '-p' to preserve permissions */
-
-#ifdef SCP_PATH
-    ag0 = SCP_PATH;
-    ag1 = "-Brp";
-#else /* SCP_PATH */
-    ag0 = RCP_PATH;
-    ag1 = "-rp";
-#endif	/* SCP_PATH */
+    ag0 = rcp_path;
+    ag1 = rcp_args;
     } 
 
   if (LOGLEVEL >= 6)
@@ -2312,9 +2467,7 @@ static int sys_copy(
 
       /* redirect stderr to make error from rcp available to MOM */
 
-      unlink(rcperr);  /* likely failing, ignore */
-
-      if ((fd = open(rcperr,O_RDWR|O_CREAT|O_EXCL,0644)) < 0) 
+      if ((fd = open(rcperr,O_RDWR|O_CREAT,0644)) < 0) 
         {
         sprintf(log_buffer,"can't open %s, error = %d",
           rcperr,errno);
@@ -2446,10 +2599,23 @@ void req_cpyfile(
   int            arg2index = -1;
   char		 faketmpdir[1024];
   job 		*pjob;
+  int		 wordexperr = 0;
 #endif
 
   if (LOGLEVEL >= 3)
     {
+    pair = (struct rqfpair *)GET_NEXT(preq->rq_ind.rq_cpyfile.rq_pair);
+
+    if ((pair != NULL) && (pair->fp_rmt != NULL))
+      {
+      sprintf(log_buffer,"attempting to copy file '%s'",
+        pair->fp_rmt);
+      }
+    else
+      {
+      sprintf(log_buffer,"copy file request is corrupt");
+      }
+
     LOG_EVENT(
       PBSEVENT_JOB,
       PBS_EVENTCLASS_JOB,
@@ -2505,6 +2671,7 @@ void req_cpyfile(
   strcat(homespool,"/.pbs_spool/");
 
   rcstat = stat(homespool,&myspooldir);
+
   if ((rcstat == 0) && S_ISDIR(myspooldir.st_mode))
     {
     havehomespool = 1;
@@ -2525,7 +2692,8 @@ void req_cpyfile(
      * has already set PBS_JOBID and HOME for us.  Now just fake a TMPDIR
      * if we need it. */
 
-    pjob=job_alloc();
+    pjob = job_alloc();
+
     strcpy(pjob->ji_qs.ji_jobid,preq->rq_ind.rq_cpyfile.rq_jobid);
 
     if (TTmpDirName(pjob,faketmpdir))
@@ -2533,8 +2701,9 @@ void req_cpyfile(
       if (!mkdirtree(faketmpdir,0755))
         {
         char *envstr;
-        envstr=malloc((strlen("TMPDIR=") + strlen(faketmpdir) + 1) * sizeof(char));
-        sprintf(envstr,"TMPDIR=%s",faketmpdir);
+        envstr = malloc((strlen("TMPDIR=") + strlen(faketmpdir) + 1) * sizeof(char));
+        sprintf(envstr,"TMPDIR=%s",
+          faketmpdir);
         putenv(envstr);
 
         madefaketmpdir = 1;
@@ -2634,7 +2803,6 @@ void req_cpyfile(
         strncpy(localname,pair->fp_local,sizeof(localname)-1);  /* from location */
         }
 
-
 #if SRFS
 
       /* Is this file part of $BIGDIR or $FASTDIR ? */
@@ -2703,7 +2871,7 @@ void req_cpyfile(
       {
       case 0:
 
-        /* NO-OP */
+        wordexperr = 0;
 
         break; /* Successful */
 
@@ -2722,6 +2890,8 @@ void req_cpyfile(
 
         bad_files = 1;
 
+        wordexperr = 1;  /* ensure we don't attempt a second source file */
+
         goto error;
 
         /*NOTREACHED*/
@@ -2735,7 +2905,20 @@ void req_cpyfile(
       {
       case 0:
 
-        break; /* Successful */
+        /* success - allow if word count is 1 */
+
+        if (arg3exp.we_wordc == 1)
+          {
+          strcpy(arg3,arg3exp.we_wordv[0]);
+
+          wordfree(&arg3exp);
+
+          wordexperr = 0;
+
+          break; /* Successful */
+          }
+
+        /* fall through */
 
       case WRDE_NOSPACE:
 
@@ -2752,30 +2935,15 @@ void req_cpyfile(
 
         bad_files = 1;
 
+        wordexperr = 1;  /* ensure we don't attempt a second destination file */
+
         goto error;
 
         break;
       }  /* END switch () */
 
-    /* Note: more than one word is only allowed for arg3 (destination) */
+    /* Note: more than one word is only allowed for arg2 (source) */
 
-    if (arg3exp.we_wordc > 1)
-      {
-      sprintf(log_buffer,"Too many words after expanding destination path: %s",
-        arg3);
-
-      add_bad_list(&bad_list,log_buffer,2);
-
-      reply_text(preq,PBSE_NOCOPYFILE,bad_list);
-
-      bad_files = 1;
-
-      goto error;
-      }
-
-    strcpy(arg3,arg3exp.we_wordv[0]);
-
-    wordfree(&arg3exp);
 
     arg2index = -1;
 
@@ -2932,7 +3100,8 @@ error:
 
     unlink(rcperr);
 #ifdef HAVE_WORDEXP
-    goto nextword;  /* ugh, it's hard to use a real loop when your feature is #ifdef's out */
+    if (!wordexperr)
+      goto nextword;  /* ugh, it's hard to use a real loop when your feature is #ifdef's out */
 #endif
     }  /* END for(pair) */
 
