@@ -121,11 +121,9 @@
 #include	"net_connect.h"
 #include	"rpp.h"
 #include	"rm.h"
-#include        "pbs_version.h"
+#include	"libpbs.h"
 
-struct		connect_handle connection[PBS_NET_MAX_CONNECTIONS];
-int		pbs_errno;
-char		pbs_current_user[PBS_MAXUSER] = "Scheduler";
+extern int	pbs_errno;
 int		connector;
 int		server_sock;
 
@@ -137,7 +135,7 @@ int             mask_num = 0;
 char		*configfile = NULL;	/* name of config file */
 
 char		*oldpath;
-char		*msg_daemonname = "pbs_sched";
+extern char		*msg_daemonname;
 char		**glob_argv;
 char		usage[] = "[-S port][-d home][-p output][-c config][-a alarm]";
 struct	sockaddr_in	saddr;
@@ -344,7 +342,7 @@ int restricted(
   {
   static  char    id[] = "restricted";
 
-  struct  hostent         *host, *gethostbyname();
+  struct  hostent         *host;
   struct  in_addr saddr;
   pbs_net_t       *newclients;
  
@@ -391,7 +389,7 @@ int addclient(
 
   {
   static char     id[] = "addclient";
-  struct hostent *host, *gethostbyname();
+  struct hostent *host;
   struct in_addr  saddr;
 
   if ((host = gethostbyname(name)) == NULL) 
@@ -578,7 +576,7 @@ int server_command()
   char	*id = "server_command";
 
   int           new_socket;
-  unsigned int	slen;
+  torque_socklen_t	slen;
   int		i, cmd;
   pbs_net_t	addr;
 
@@ -711,6 +709,9 @@ int main(
 	pbs_rm_port = get_svrport(PBS_MANAGER_SERVICE_NAME, "tcp",
 			   PBS_MANAGER_SERVICE_PORT);
 
+	strcpy(pbs_current_user,"Scheduler");
+	msg_daemonname=strdup("pbs_sched");
+
 	opterr = 0;
 
         while ((c = getopt(argc, argv, "L:S:R:d:p:c:a:-:")) != EOF) {
@@ -721,7 +722,7 @@ int main(
                         }
 
                         if (!strcmp(optarg,"version")) {
-                                fprintf(stderr,"version: %s\n", PBS_VERSION);
+                                fprintf(stderr,"version: %s\n", PACKAGE_VERSION);
                                 exit(0);
                         } else {
                                 errflg = 1;
@@ -991,7 +992,12 @@ int main(
 
         (void)sprintf(log_buffer, "%ld\n", (long)pid);
 
-        (void)write(lockfds, log_buffer, strlen(log_buffer)+1);
+        if (write(lockfds, log_buffer, strlen(log_buffer)+1) != (ssize_t)(strlen(log_buffer)+1))
+          {
+          perror("writing to lockfile");
+
+          exit(1);
+          }
 
 #if (PLOCK_DAEMONS & 2)
 	(void)plock(PROCLOCK);	/* lock daemon into memory */
