@@ -103,6 +103,10 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #include <time.h>
+#if defined(NTOHL_NEEDS_ARPA_INET_H) && defined(HAVE_ARPA_INET_H)
+#include <arpa/inet.h>
+#endif
+
 #include "pbs_ifl.h"
 #include "libpbs.h"
 #include "list_link.h"
@@ -178,7 +182,7 @@ struct pbsnode *PGetNodeFromAddr(
   for (nindex = 0;nindex < svr_totnodes;nindex++)
     {
     if ((pbsndlist[nindex] == NULL) || 
-        (pbsndlist[nindex]->nd_addrs == NULL))
+        (pbsndlist[nindex]->nd_state & INUSE_DELETED))
       continue;
 
     for (aindex = 0;aindex < 10;aindex++)
@@ -223,39 +227,54 @@ void bad_node_warning(
       continue;
       }
 
-    if (pbsndlist[i]->nd_addrs[0] == addr) 
+    if (pbsndlist[i]->nd_state & INUSE_DELETED)
       {
-      now = time(0);
+      /* node was deleted */
 
-      last = pbsndlist[i]->nd_warnbad;
+      continue;
+      }
+
+    if (pbsndlist[i]->nd_addrs[0] != addr) 
+      {
+      /* node does not match */
+
+      continue;
+      }
+
+    /* matching node located */
+
+    now = time(0);
+
+    last = pbsndlist[i]->nd_warnbad;
  
-      if (last && (now - last < 3600))
-        {
-        return;
-        }
+    if (last && (now - last < 3600))
+      {
+      /* bad node warning already sent within the last hour */
 
-      /*
-      ** once per hour, log a warning that we can't reach the node, and
-      ** ping_nodes to check and reset the node's state.
-      */
+      return;
+      }
 
-      sprintf(log_buffer,"ALERT: unable to contact node %s",
-        pbsndlist[i]->nd_name);
+    /*
+    ** once per hour, log a warning that we can't reach the node, and
+    ** ping_nodes to check and reset the node's state.
+    */
 
-      log_event(
-        PBSEVENT_ADMIN, 
-        PBS_EVENTCLASS_SERVER, 
-        "WARNING",
-        log_buffer);
+    sprintf(log_buffer,"ALERT: unable to contact node %s",
+      pbsndlist[i]->nd_name);
 
-      /*
-       *  (void)set_task(WORK_Timed,now + 5,ping_nodes,NULL);
-       */
+    log_event(
+      PBSEVENT_ADMIN, 
+      PBS_EVENTCLASS_SERVER, 
+      "WARNING",
+      log_buffer);
 
-      pbsndlist[i]->nd_warnbad = now;
+    /*
+     *  (void)set_task(WORK_Timed,now + 5,ping_nodes,NULL);
+     */
 
-      break;
-      }  /* END if (pbsndlist[i]->nd_addrs[0] == addr) */
+    pbsndlist[i]->nd_warnbad = now;
+
+    break;
     }    /* END for (i = 0) */
 
   return;
@@ -283,9 +302,17 @@ int addr_ok(
       {
       /* NOTE:  should walk thru all nd_addrs for multi-homed hosts */
 
-      if (pbsndlist[i]->nd_addrs[0] != addr)
+      if (pbsndlist[i]->nd_state & INUSE_DELETED)
         continue;
 
+      /* NOTE:  deleted node may have already freed nd_addrs - 
+                check should be redundant */
+
+      if ((pbsndlist[i]->nd_addrs == NULL) || (pbsndlist[i]->nd_addrs[0] != addr))
+        continue;
+
+      /* node matches addr */
+
       if (pbsndlist[i]->nd_state & (INUSE_DELETED|INUSE_UNKNOWN))
         {
         /* definitely not ok */
@@ -295,9 +322,11 @@ int addr_ok(
       else if (pbsndlist[i]->nd_state & INUSE_DOWN)
         {
         /* the node is ok if it is still talking to us */
+
         int chk_len;
 
         chk_len = server.sv_attr[(int)SRV_ATR_check_rate].at_val.at_long;
+
         if (pbsndlist[i]->nd_lastupdate <= (time_now - chk_len))
           {
           status = 0;
@@ -311,7 +340,7 @@ int addr_ok(
 
       break;
       }
-    }
+    }    /* END if (pbsndlist != NULL) */
 
   return(status);
   }  /* END addr_ok() */
@@ -540,17 +569,17 @@ int chk_characteristic(
 
 int status_nodeattrib(
 
-  svrattrl        *pal,	/*an svrattrl from the request  */
+  svrattrl        *pal,	        /*an svrattrl from the request  */
   attribute_def   *padef,	/*the defined node attributes   */
   struct pbsnode  *pnode,	/*no longer an attribute ptr	*/
   int              limit,	/*number of array elts in padef */
   int              priv,	/*requester's privilege		*/
 
-  list_head       *phead,	/*heads list of svrattrl structs that hang */
+  tlist_head       *phead,	/*heads list of svrattrl structs that hang */
 				/*off the brp_attr member of the status sub*/
 				/*structure in the request's "reply area"  */
 
-  int             *bad)	/*if node-attribute error record it's*/
+  int             *bad)	        /*if node-attribute error, record it's*/
 				/*list position here                 */
   {
   int   i;
@@ -746,20 +775,27 @@ static void initialize_pbsnode(
  *	by marking it deleted
  */
 
-static void subnode_delete(psubn)
-	struct pbssubn *psubn;
-{
-	struct jobinfo	*jip, *jipt;
+static void subnode_delete(
 
-	for (jip = psubn->jobs; jip; jip = jipt) {
-       		jipt = jip->next;
-       		free (jip); 
-	}
-	psubn->host  = (struct pbsnode *)0;
-	psubn->jobs  = (struct jobinfo *)0;
-	psubn->next  = (struct pbssubn *)0;
-	psubn->inuse = INUSE_DELETED; 
-}
+  struct pbssubn *psubn)
+
+  {
+  struct jobinfo *jip, *jipt;
+
+  for (jip = psubn->jobs;jip;jip = jipt) 
+    {
+    jipt = jip->next;
+
+    free(jip); 
+    }
+
+  psubn->host  = NULL;
+  psubn->jobs  = NULL;
+  psubn->next  = NULL;
+  psubn->inuse = INUSE_DELETED; 
+
+  return;
+  }
 
 
 
@@ -775,7 +811,7 @@ void effective_node_delete(
 
   psubn = pnode->nd_psn;
 
-  while (psubn) 
+  while (psubn != NULL) 
     {
     pnxt = psubn->next;
 
@@ -784,25 +820,30 @@ void effective_node_delete(
     psubn = pnxt;
     }
 
-  pnode->nd_last->next = (struct prop *)0; /*just in case*/
-  pnode->nd_last = (struct prop *)0;
+  pnode->nd_last->next = NULL;      /* just in case */
+  pnode->nd_last       = NULL;
 
-  free_prop_list (pnode->nd_first);
+  free_prop_list(pnode->nd_first);
 
-  pnode->nd_first = (struct prop *)0;
+  pnode->nd_first = NULL;
 
-  for (up = pnode->nd_addrs;*up != 0;up++) 
+  if (pnode->nd_addrs != NULL)
     {
-    /* del node's IP addresses from tree  */
+    for (up = pnode->nd_addrs;*up != 0;up++) 
+      {
+      /* del node's IP addresses from tree  */
 
-    tdelete(*up,&ipaddrs);
-    }
+      tdelete(*up,&ipaddrs);
+      }
 
-  if (pnode->nd_addrs != NULL) 
-    {	/* remove array of IP addresses */
-    free(pnode->nd_addrs);
+    if (pnode->nd_addrs != NULL) 
+      {	
+      /* remove array of IP addresses */
 
-    pnode->nd_addrs = NULL;
+      free(pnode->nd_addrs);
+
+      pnode->nd_addrs = NULL;
+      }
     }
 
   tdelete((u_long)pnode->nd_stream,&streams); /*take stream out of tree*/
@@ -810,41 +851,17 @@ void effective_node_delete(
   rpp_close(pnode->nd_stream);
   free(pnode->nd_name);
 
-  pnode->nd_name = NULL;
-  pnode->nd_stream = -1;
-  pnode->nd_state  = INUSE_DELETED;
-  pnode->nd_nsn    = 0;
-  pnode->nd_nsnfree= 0;
+  pnode->nd_name    = NULL;
+  pnode->nd_stream  = -1;
+  pnode->nd_state   = INUSE_DELETED;
+  pnode->nd_nsn     = 0;
+  pnode->nd_nsnfree = 0;
 
   return;
-  }
+  }  /* END effective_node_delete() */
 
 
 
-/*
- *	setup_notification -  Sets up the  mechanism for notifying
- *			      other members of the server's node
- *			      pool that a new node was added manually
- *			      via qmgr.  Actual notification occurs some
- *			      time later through the ping_nodes mechanism
-*/
-
-void setup_notification()
-
-  {
-  int i;
-
-  for (i = 0;i < svr_totnodes;i++) 
-    {
-    if (pbsndlist[i]->nd_state & INUSE_DELETED)
-      continue;
-
-    pbsndlist[i]->nd_state |= INUSE_NEEDS_HELLO_PING;
-    }
-
-  return;
-  }
-
 
 
 
@@ -1155,7 +1172,7 @@ struct prop *init_prop(
 /*
  * create_subnode - create a subnode entry and link to parent node
  *
- *	Note, pname arg must be a copy of prop list as it is linked directly in
+ *  NOTE: pname arg must be a copy of prop list as it is linked directly in
  */
 
 static struct pbssubn *create_subnode(
@@ -1168,30 +1185,35 @@ static struct pbssubn *create_subnode(
 	
   psubn = (struct pbssubn *)malloc(sizeof(struct pbssubn));
 
-  if (psubn == (struct pbssubn *)0) 
+  if (psubn == NULL) 
     {
     return(NULL);
     }
 
-	/* initialize the subnode and link into the parent node */
+/* initialize the subnode and link into the parent node */
 
-	psubn->host  = pnode;
-	psubn->next  = (struct pbssubn *)NULL;
-	psubn->jobs  = (struct jobinfo *)NULL;
-	psubn->flag  = okay;
-	psubn->inuse = 0;
-	psubn->index = pnode->nd_nsn++;
-	pnode->nd_nsnfree++;
-	if ((pnode->nd_state & (INUSE_JOB|INUSE_JOBSHARE)) != 0)
-		pnode->nd_state = INUSE_FREE;
-	psubn->allocto = (resource_t)0;
+  psubn->host  = pnode;
+  psubn->next  = NULL;
+  psubn->jobs  = NULL;
+  psubn->flag  = okay;
+  psubn->inuse = 0;
+  psubn->index = pnode->nd_nsn++;
 
-	nxtsn = &pnode->nd_psn;	   /* link subnode onto parent node's list */
-	while (*nxtsn)
-		nxtsn = &((*nxtsn)->next);
-	*nxtsn = psubn;
+  pnode->nd_nsnfree++;
 
-	return (psubn);
+  if ((pnode->nd_state & (INUSE_JOB|INUSE_JOBSHARE)) != 0)
+    pnode->nd_state = INUSE_FREE;
+
+  psubn->allocto = (resource_t)0;
+
+  nxtsn = &pnode->nd_psn;	   /* link subnode onto parent node's list */
+
+  while (*nxtsn)
+    nxtsn = &((*nxtsn)->next);
+
+  *nxtsn = psubn;
+
+  return(psubn);
   }  /* END create_subnode() */
 
 
@@ -1220,7 +1242,7 @@ int create_pbs_node(
 
   if ((rc = process_host_name_part(objname,&pul,&pname,&ntype)) != 0) 
     {
-    log_err(-1, "process_host_name_part", log_buffer);
+    log_err(-1,"process_host_name_part",log_buffer);
 
     return(rc);
     }
@@ -1438,7 +1460,7 @@ int setup_nodes(void)
   char      xchar;
   svrattrl *pal;
   int	  perm = ATR_DFLAG_MGRD|ATR_DFLAG_MGWR;
-  list_head atrlist;
+  tlist_head atrlist;
 
   extern char server_name[];
   extern resource_t next_resource_tag;
@@ -1456,7 +1478,7 @@ int setup_nodes(void)
 
   if ((nin = fopen(path_nodes,"r")) == NULL) 
     {
-    sprintf(log_buffer,"cannot open node description file '%s' in setup_nodes()\n",
+    sprintf(log_buffer,"cannot open node description file '%s' in setup_nodes()",
       path_nodes);
 
     log_event(
@@ -1658,7 +1680,10 @@ int setup_nodes(void)
 
         if (strcmp(np->nd_name,line) == 0) 
           {
-          np->nd_state = num;
+          np->nd_state = num|INUSE_NEEDS_HELLO_PING;
+
+          /* exclusive bits are calculated later in set_old_nodes() */
+          np->nd_state &= ~(INUSE_JOB|INUSE_JOBSHARE);
 
           break;
           }
