@@ -121,7 +121,6 @@
 #include "dis.h"
 #include "dis_init.h"
 #include "batch_request.h"
-#include "pbs_version.h"
 #include "pbs_proto.h"
 
 /* external functions called */
@@ -131,15 +130,18 @@ extern void shutdown_ack ();
 extern void update_nodes_file A_((void));
 extern void tcp_settimeout(long);
 extern void poll_job_task(struct work_task *);
-extern int schedule_jobs(void);
+extern int  schedule_jobs(void);
 extern void queue_route A_((pbs_queue *));
 extern void svr_shutdown(int);
 extern void acct_close(void);
-extern int  svr_startjob A_((job *,struct batch_request *)); 
+extern int  svr_startjob A_((job *,struct batch_request *,char *,char *)); 
+extern int RPPConfigure(int,int);
 
 
 /* external data items */
-extern	int	svr_chngNodesfile;
+
+extern int    svr_chngNodesfile;
+extern int    svr_totnodes;
 
 
 /* Local Private Functions */
@@ -155,7 +157,7 @@ char	       *acct_file = NULL;
 char	       *log_file  = NULL;
 char	       *path_home = PBS_SERVER_HOME;
 char	       *path_acct;
-char	        path_log[MAXPATHLEN+1];
+char	        path_log[MAXPATHLEN + 1];
 char	       *path_priv;
 char	       *path_jobs;
 char	       *path_queues;
@@ -166,16 +168,15 @@ char 	       *path_track;
 char	       *path_nodes;
 char	       *path_nodes_new;
 char	       *path_nodestate;
-char		pbs_current_user[PBS_MAXUSER]="PBS_Server"; /* for libpbs.a */
-char	       *msg_daemonname = pbs_current_user;
-int		pbs_errno;
+extern char    *msg_daemonname;
+extern int	pbs_errno;
 char	       *pbs_o_host = "PBS_O_HOST";
 pbs_net_t	pbs_mom_addr;
 unsigned int	pbs_mom_port;
 unsigned int	pbs_rm_port;
 pbs_net_t	pbs_scheduler_addr;
 unsigned int	pbs_scheduler_port;
-pbs_net_t	pbs_server_addr;
+extern pbs_net_t	pbs_server_addr;
 unsigned int	pbs_server_port_dis;
 int		queue_rank = 0;
 struct server	server;		/* the server structure */
@@ -185,19 +186,19 @@ int	 	server_init_type = RECOV_WARM;
 char	        server_name[PBS_MAXSERVERNAME + 1]; /* host_name[:service|port] */
 int		svr_delay_entry = 0;
 int		svr_do_schedule = SCH_SCHEDULE_NULL;
-list_head	svr_queues;            /* list of queues                   */
-list_head	svr_alljobs;           /* list of all jobs in server       */
-list_head	svr_newjobs;           /* list of incoming new jobs        */
-list_head	task_list_immed;
-list_head	task_list_timed;
-list_head	task_list_event;
+tlist_head	svr_queues;            /* list of queues                   */
+tlist_head	svr_alljobs;           /* list of all jobs in server       */
+tlist_head	svr_newjobs;           /* list of incoming new jobs        */
+tlist_head	svr_newnodes;          /* list of newly created nodes      */
+tlist_head	task_list_immed;
+tlist_head	task_list_timed;
+tlist_head	task_list_event;
 
 time_t		time_now = 0;
 
 int             LOGLEVEL = 0;
 int             DEBUGMODE = 0;
-int             TAllowComputeHostSubmit = FALSE;
-char          **TAllowSubmitHostList = NULL;
+int             TForceUpdate = 0;  /* (boolean) */
 
 char           *ProgName;
 
@@ -235,13 +236,14 @@ void do_rpp(
   {
   static char id[] = "do_rpp";
 
-  int	ret, proto, version;
-  void	is_request A_((int,int,int *));
-  void	stream_eof A_((int,u_long,int));
+  int  ret, proto, version;
+
+  void is_request A_((int,int,int *));
+  void stream_eof A_((int,u_long,int));
 
   if (LOGLEVEL >= 4)
     {
-    sprintf(log_buffer,"rpp request received on stream %d\n",
+    sprintf(log_buffer,"rpp request received on stream %d",
       stream);
 
     log_record(
@@ -257,6 +259,18 @@ void do_rpp(
 
   if (ret != DIS_SUCCESS) 
     {
+    if (LOGLEVEL >= 1)
+      {
+      sprintf(log_buffer,"corrupt rpp request received on stream %d (invalid protocol)",
+        stream);
+
+      log_record(
+        PBSEVENT_SCHED,
+        PBS_EVENTCLASS_REQUEST,
+        id,
+        log_buffer);
+      }
+
     stream_eof(stream,0,ret);
 
     return;
@@ -266,6 +280,18 @@ void do_rpp(
 
   if (ret != DIS_SUCCESS) 
     {
+    if (LOGLEVEL >= 1)
+      {
+      sprintf(log_buffer,"corrupt rpp request received on stream %d (invalid version)",
+        stream);
+
+      log_record(
+        PBSEVENT_SCHED,
+        PBS_EVENTCLASS_REQUEST,
+        id,
+        log_buffer);
+      }
+
     stream_eof(stream,0,ret);
 
     return;
@@ -277,7 +303,7 @@ void do_rpp(
 
       if (LOGLEVEL >= 6)
         {
-        sprintf(log_buffer,"inter-server request received\n");
+        sprintf(log_buffer,"inter-server request received");
 
         log_record(
           PBSEVENT_SCHED,
@@ -346,7 +372,7 @@ int PBSShowUsage(
   char *EMsg)  /* I (optional) */
 
   {
-  const char *Msg = "[ -A <ACCTFILE> ] [ -a <ATTR> ] [ -d <HOMEDIR> ] [ -L <LOGFILE> ] [ -M <MOMPORT> ]\n  [ -p <SERVERPORT> ] [ -R <RMPORT> ] [ -S <SCHEDULERPORT> ] [ -t <TYPE> ]\n  [ --version ]\n";
+  const char *Msg = "[ -A <ACCTFILE> ] [ -a <ATTR> ] [ -d <HOMEDIR> ] [ -L <LOGFILE> ] [ -M <MOMPORT> ]\n  [ -p <SERVERPORT> ] [ -R <RMPORT> ] [ -S <SCHEDULERPORT> ] [ -t <TYPE> ]\n  [ --version|--help ]\n";
 
   fprintf(stderr,"usage:  %s %s\n",
     ProgName,
@@ -364,64 +390,6 @@ int PBSShowUsage(
 
 
 
-#define TCONST_CFGFILE "torque.cfg"
-
-int TLoadConfig(
-
-  char *Buffer,    /* O */
-  int   BufSize)   /* I */
-
-  {
-  FILE *config_stream;
-
-  char home_dir[MAXPATHLEN];
-
-  int  length = strlen(PBS_SERVER_HOME) + strlen(TCONST_CFGFILE) + 2;
-
-  if (Buffer == NULL)
-    {
-    /* FAILURE */
-
-    return(1);
-    }
-
-  if (length >= MAXPATHLEN)
-    {
-    /* FAILURE */
-
-    return(1);
-    }
-
-  home_dir[0] = '\0';
-
-  strcat(home_dir,PBS_SERVER_HOME);
-
-  strcat(home_dir,"/");
-
-  strcat(home_dir,TCONST_CFGFILE);
-
-  if ((config_stream = fopen(home_dir,"r")) == NULL)
-    {
-    /* FAILURE */
-
-    return(1);
-    }
-
-  if ((fread(Buffer,BufSize,1,config_stream) <= 0) && (ferror(config_stream) != 0))
-    {
-    /* FAILURE */
-
-    return(1);
-    }
-
-  fclose(config_stream);
-
-  /* SUCCESS */
-
-  return(0);
-  }  /* END TLoadConfig() */
-
-
 
 /*
  * main - the initialization and main loop of pbs_daemon
@@ -452,12 +420,9 @@ int main(
   time_t last_jobstat_time;
   int    when;
 
-  int    BufSize;
-
-  char   *Buffer;
-
   void	 ping_nodes A_((struct work_task *ptask));
   void   check_nodes A_((struct work_task *ptask));
+  void   check_log A_((struct work_task *ptask));
 
   static struct {
     char *it_name;
@@ -469,19 +434,16 @@ int main(
    { "create",	RECOV_CREATE },
    { "",	RECOV_Invalid } };
 
-  extern  int   optind;
-  extern	char *optarg;
-  extern  char *msg_daemonname;
-  extern  char *msg_svrdown;	/* log message   */
-  extern  char *msg_startup1;	/* log message   */
-  extern  char *msg_startup2;	/* log message   */
+  extern int   optind;
+  extern char *optarg;
+  extern char *msg_svrdown;	/* log message   */
+  extern char *msg_startup1;	/* log message   */
+  extern char *msg_startup2;	/* log message   */
 
   ProgName = argv[0];
 
-  BufSize = 65536;
-
-  Buffer = calloc(1,BufSize);
-
+  strcpy(pbs_current_user,"PBS_Server");
+  msg_daemonname=strdup(pbs_current_user);
 
   /* if we are not running with real and effective uid of 0, forget it */
 
@@ -505,97 +467,6 @@ int main(
 
   server_host[0] = '\0';
 
-  /* load/process config file first then override values with command line parameters */
-
-  if (TLoadConfig(Buffer,BufSize) == 0)
-    {
-    char *ptr;
-    char *tptr;
-
-    char *ltokptr;  /* line token pointer */
-    char *vtokptr;  /* value token pointer */
-
-    /* success - process config file */
-
-    ptr = strtok_r(Buffer,"\n",&ltokptr);
-
-    while (ptr != NULL)
-      {
-      if (ptr[0] == '#')
-        {
-        /* ignore comments */
-
-        ptr = strtok_r(NULL,"\n",&ltokptr);
-
-        continue;
-        }
-
-      if (!strncmp(ptr,"SERVERHOST",strlen("SERVERHOST")))
-        {
-        tptr = ptr + strlen("SERVERHOST");
-
-        while (isspace(*tptr) && (*tptr != '\0'))
-          tptr++;
-
-        if (sscanf(tptr,"%64s",server_host) != 1)
-          {
-          log_err(-1,"pbsd_main","parse error in config at SERVERHOST");
-
-          return(-1);
-          }
-        }
-      else if (!strncmp(ptr,"ALLOWCOMPUTEHOSTSUBMIT",strlen("ALLOWCOMPUTEHOSTSUBMIT")))
-        {
-        /* FORMAT:  ALLOWCOMPUTEHOSTSUBMIT  true */
-
-        tptr = ptr + strlen("ALLOWCOMPUTEHOSTSUBMIT");
-
-        while (isspace(*tptr) && (*tptr != '\0'))
-          tptr++;
-
-        if (!strncasecmp(tptr,"true",strlen("true")) ||
-            !strncasecmp(tptr,"on",strlen("on")) ||
-            !strncasecmp(tptr,"yes",strlen("yes")) ||
-            (*tptr == '1'))
-          {
-          TAllowComputeHostSubmit = TRUE;
-          }
-        }
-      else if (!strncmp(ptr,"SUBMITHOSTS",strlen("SUBMITHOSTS")))
-        {
-        int hcount;
-
-        /* FORMAT:  SUBMITHOSTS  <HOST>[,<HOST>]... */
-
-        tptr = ptr + strlen("SUBMITHOSTS");
-
-        TAllowSubmitHostList = (char **)calloc(1,sizeof(char *) * (2048 + 1));
-
-        hcount = 0;
-
-        tptr = strtok_r(tptr,"+,: \t\n",&vtokptr);
-
-        while (tptr != NULL)
-          {
-          TAllowSubmitHostList[hcount] = strdup(tptr);
-
-          hcount++;
-
-          if (hcount >= 2048)
-            break;
-
-          tptr = strtok_r(NULL,"+,: \t\n",&vtokptr);
-          }  /* END while (tptr != NULL) */
-
-        TAllowSubmitHostList[hcount] = NULL;
-        }  /* END else if (!strncmp(ptr,"SUBMITHOSTS",strlen("SUBMITHOSTS"))) */
-
-      ptr = strtok_r(NULL,"\n",&ltokptr);
-      }  /* END while (ptr != NULL) */
-    }    /* END if (TLoadConfig(Buffer,sizeof(Buffer)) == 0) */
-
-  free(Buffer);
-
   if (((server_host[0] == '\0') && (gethostname(server_host,PBS_MAXHOSTNAME) == -1)) ||
        (get_fullhostname(server_host,server_host,PBS_MAXHOSTNAME) == -1)) 
     {
@@ -626,7 +497,7 @@ int main(
 
   /* parse the parameters from the command line */
 
-  while ((c = getopt(argc,argv,"A:a:d:h:p:t:L:M:R:S:-:")) != -1) 
+  while ((c = getopt(argc,argv,"A:a:d:fh:p:t:L:M:R:S:-:")) != -1) 
     {
     switch (c) 
       {
@@ -634,16 +505,44 @@ int main(
 
         if ((optarg == NULL) || (optarg[0] == '\0'))
           {
-          exit(0);
+          PBSShowUsage("empty command line arg");
+
+          exit(1);
           }
 
         if (!strcmp(optarg,"version"))
           {
           fprintf(stderr,"version: %s\n",
-            PBS_VERSION);
+            PACKAGE_VERSION);
 
           exit(0);
           }
+        else if (!strcmp(optarg,"about"))
+          {
+	  printf("package: %s\n",PACKAGE_STRING);
+	  printf("sourcedir %s\n",PBS_SOURCE_DIR);
+	  printf("configure: %s\n",PBS_CONFIG_ARGS);
+	  printf("buildcflags: %s\n",PBS_CFLAGS);
+	  printf("buildhost: %s\n",PBS_BUILD_HOST);
+	  printf("builddate: %s\n",PBS_BUILD_DATE);
+	  printf("buildir: %s\n",PBS_BUILD_DIR);
+	  printf("builduser: %s\n",PBS_BUILD_USER);
+	  printf("installdir: %s\n",PBS_INSTALL_DIR);
+	  printf("serverhome: %s\n",PBS_SERVER_HOME);
+          exit(0);
+          }
+        else if (!strcmp(optarg,"help"))
+          {
+          PBSShowUsage(NULL);
+    
+          exit(1);
+          }
+        else
+          {
+          PBSShowUsage("invalid command line arg");
+
+          exit(1);
+          }
 
         break;
 
@@ -669,6 +568,12 @@ int main(
 
         break;
 
+      case 'f':
+
+        TForceUpdate = 1;
+ 
+        break;
+
       case 'h':
 
         /* overwrite locally detected hostname with specified hostname */
@@ -892,6 +797,7 @@ int main(
 
   log_open(log_file,path_log);
 
+
   sprintf(log_buffer,msg_startup1,server_name,server_init_type);
 
   log_event(
@@ -1034,6 +940,15 @@ int main(
     return(1);
     }
 
+  if (LOGLEVEL >= 5)
+    {
+    log_event(
+      PBSEVENT_SYSTEM | PBSEVENT_FORCE,
+      PBS_EVENTCLASS_SERVER,
+      msg_daemonname,
+      "creating rpp and private interfaces");
+    }
+
   add_conn(rppfd,Primary,(pbs_net_t)0,0,rpp_request);
   add_conn(privfd,Primary,(pbs_net_t)0,0,rpp_request);
 
@@ -1050,11 +965,24 @@ int main(
   /* do not check nodes immediately as they will initially be marked 
      down unless they have already reported in */
 
-  set_task(WORK_Immed,time_now + 60,check_nodes,NULL);
+  when = server.sv_attr[(int)SRV_ATR_check_rate].at_val.at_long + time_now;
+  if (svr_totnodes > 1024)
+    {
+    /* for large systems, give newly reported nodes more time before 
+       being marked down while pbs_moms are intialy reporting in */
+
+    set_task(WORK_Timed,when + svr_totnodes / 12,check_nodes,NULL);
+    }
+  else
+    {
+    set_task(WORK_Timed,when,check_nodes,NULL);
+    }
 
   /* Just check the nodes with check_nodes above and don't ping anymore. */
 
-  set_task(WORK_Immed,0,ping_nodes,NULL); 
+  set_task(WORK_Timed,time_now + 5,ping_nodes,NULL);
+
+  set_task(WORK_Timed,time_now + 5,check_log,NULL);
 
   /*
    * Now at last, we are ready to do some batch work.  The
@@ -1070,6 +998,8 @@ int main(
   else
     *state = SV_STATE_RUN;
 
+  DBPRT(("pbs_server is up\n"));
+
   while (*state != SV_STATE_DOWN) 
     {
     /* first process any task whose time delay has expired */
@@ -1151,12 +1081,15 @@ int main(
 
     if (!getenv("PBSLOGLEVEL"))
       LOGLEVEL = server.sv_attr[(int)SRV_ATR_LogLevel].at_val.at_long;
+    
 
     /* any running jobs need a status update? */ 
 
     if (server.sv_attr[(int)SRV_ATR_PollJobs].at_val.at_long && 
        (last_jobstat_time + JobStatRate <= time_now))
       {
+      struct work_task *ptask;
+
       for (pjob = (job *)GET_NEXT(svr_alljobs);
            pjob != NULL;
            pjob = (job *)GET_NEXT(pjob->ji_alljobs)) 
@@ -1167,12 +1100,18 @@ int main(
 
           when = pjob->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long % JobStatRate;
     
-          set_task(WORK_Timed,when + time_now,poll_job_task,pjob);
+          ptask = set_task(WORK_Timed,when + time_now,poll_job_task,pjob);
+
+          if (ptask)
+            {
+            append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
+            }
+
           }
         }
 
       last_jobstat_time = time_now;
-      }  /* END if (...) */
+      }  /* END if (should poll jobs now) */
 
     if (*state == SV_STATE_SHUTSIG) 
       svr_shutdown(SHUT_SIG);	/* caught sig */
@@ -1213,6 +1152,7 @@ int main(
     update_nodes_file();
     }
 
+  RPPConfigure(1,0);  /* help rpp_shutdown go a bit faster */
   rpp_shutdown();
 
   shutdown_ack();
@@ -1233,6 +1173,42 @@ int main(
   }  /* END main() */
 
 
+void check_log(
+
+ struct work_task *ptask)
+
+ {
+ long depth = 1;
+
+ if ((server.sv_attr[(int)SRV_ATR_LogFileMaxSize].at_flags 
+      & ATR_VFLAG_SET) != 0)
+   {
+   if (log_size() >= server.sv_attr[(int)SRV_ATR_LogFileMaxSize].at_val.at_long)
+     {
+     log_event(
+       PBSEVENT_SYSTEM | PBSEVENT_FORCE,
+       PBS_EVENTCLASS_SERVER,
+       msg_daemonname,
+       "Rolling log file");
+
+      if ((server.sv_attr[(int)SRV_ATR_LogFileRollDepth].at_flags 
+           & ATR_VFLAG_SET) != 0)
+        {
+        depth = server.sv_attr[(int)SRV_ATR_LogFileRollDepth].at_val.at_long;
+        }
+
+      if ((depth >= INT_MAX) || (depth < 1))
+        {
+        log_err(-1,"check_log","log roll cancelled, logfile depth is out of range");
+        }
+      else
+        {
+        log_roll(depth);
+        }
+      }
+    }
+ set_task(WORK_Timed,time_now + 300, check_log, NULL);   
+ } /* END check_log */
 
 
 
@@ -1382,7 +1358,7 @@ static int start_hot_jobs(void)
         pjob->ji_qs.ji_jobid, 
         "attempting to hot start job");
 
-      svr_startjob(pjob,NULL);
+      svr_startjob(pjob,NULL,NULL,NULL);
 
       ct++;
       }
