@@ -574,6 +574,7 @@ int mom_comm(
 
   {
   unsigned int dummy;
+  struct work_task *pwt;
 
   if (pjob->ji_momhandle < 0) 
     {
@@ -593,12 +594,19 @@ int mom_comm(
 
     if (pjob->ji_momhandle < 0) 
       {
-      set_task(
+      pwt = set_task(
         WORK_Timed,
         (long)(time_now + PBS_NET_RETRY_TIME), 
         func, 
         (void *)pjob);
 
+      if (pwt)
+        {
+        /* insure that work task will be removed if job goes away */
+
+        append_link(&pjob->ji_svrtask,&pwt->wt_linkobj,pwt);
+        }
+
       return(-1);
       }
     }
@@ -613,7 +621,7 @@ int mom_comm(
  * rel_resc - release resources assigned to the job
  */
 
-static void rel_resc(
+void rel_resc(
 
   job *pjob)  /* I (modified) */
 
@@ -651,13 +659,15 @@ static void rel_resc(
  *	rq_extra field in the request points to the job.
  */
 
+extern tlist_head svr_alljobs;
+
 void on_job_exit(
 
   struct work_task *ptask)  /* I */
 
   {
   int    handle;
-  job   *pjob;
+  job   *pjob, *pj;
   struct batch_request *preq;
 
   int    IsFaked = 0;
@@ -679,6 +689,38 @@ void on_job_exit(
     pjob = (job *)preq->rq_extra;
     }
 
+#ifdef VNODETESTING
+/* FIXME: there might be a race with calling on_job_exit after a job has
+ * already been free'd.  This is temp code */
+  pj = (job *)GET_NEXT(svr_alljobs);
+  
+  while (pj != NULL)
+    {
+    if (pjob==pj)
+      break;
+    
+    pj = (job *)GET_NEXT(pj->ji_alljobs);
+    }
+  
+  if (pj==NULL)
+    {
+    sprintf(log_buffer,"on_job_exit called with INVALID pjob: %p",pjob);
+    }
+  else
+    {
+    sprintf(log_buffer,"on_job_exit valid pjob: %p (substate=%d)",pjob,pjob->ji_qs.ji_substate);
+    }
+
+  log_event(
+    PBSEVENT_JOB,
+    PBS_EVENTCLASS_JOB,
+    pjob->ji_qs.ji_jobid,
+    log_buffer);
+
+  DBPRT(("%s\n",log_buffer));
+#endif
+
+
   if ((handle = mom_comm(pjob, on_job_exit)) < 0)
     {
     return;
@@ -758,6 +800,11 @@ void on_job_exit(
 
           ptask = set_task(WORK_Immed,0,on_job_exit,pjob);
 
+          if (ptask)
+            {
+            append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
+            }
+
           return;
           }
         }    /* END if (ptask->wt_type != WORK_Deferred_Reply) */
@@ -881,6 +928,11 @@ void on_job_exit(
 
           ptask = set_task(WORK_Immed,0,on_job_exit,pjob);
 
+          if (ptask)
+            {
+            append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
+            }
+
           return;
           }
         }
@@ -959,6 +1011,11 @@ void on_job_exit(
 
       svr_setjobstate(pjob,JOB_STATE_COMPLETE,JOB_SUBSTATE_COMPLETE);
 
+      if ((pque = pjob->ji_qhdr) && (pque != NULL))
+        {
+        pque->qu_numcompleted++;
+        }
+
       ptask->wt_type = WORK_Immed;
 
       /* NO BREAK, FALL INTO NEXT CASE */
@@ -967,7 +1024,9 @@ void on_job_exit(
 
       if ((pque = pjob->ji_qhdr) && (pque->qu_attr != NULL))
         {
-        KeepSeconds = (int)pque->qu_attr[(int)QE_ATR_KeepCompleted].at_val.at_long;
+        KeepSeconds = attr_ifelse_long(&pque->qu_attr[(int)QE_ATR_KeepCompleted],
+                                       &server.sv_attr[(int)SRV_ATR_KeepCompleted],
+                                       0);
         }
 
       if (KeepSeconds <= 0)
@@ -1062,6 +1121,11 @@ void on_job_rerun(
 
           ptask = set_task(WORK_Immed,0,on_job_rerun,pjob);
 
+          if (ptask)
+            {
+            append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
+            }
+
           return;
           }
 			
@@ -1180,6 +1244,11 @@ void on_job_rerun(
 
           ptask = set_task(WORK_Immed,0,on_job_rerun,pjob);
 
+          if (ptask)
+            {
+            append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
+            }
+
           return;
           }
         }
@@ -1275,6 +1344,11 @@ void on_job_rerun(
 
           ptask = set_task(WORK_Immed,0,on_job_rerun,pjob);
 
+          if (ptask)
+            {
+            append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
+            }
+
           return;
           }
         }
@@ -1441,12 +1515,15 @@ void req_jobobit(
   job		 *pjob;
   struct work_task *ptask;
   svrattrl	 *patlist;
+  unsigned int dummy;
 
   pjob = find_job(preq->rq_ind.rq_jobobit.rq_jid);
 
-  if (pjob == NULL) 
+  if ((pjob == NULL) ||
+      (pjob->ji_qs.ji_un.ji_exect.ji_momaddr != get_hostaddr(
+              parse_servername(preq->rq_host,&dummy)))) 
     {
-    /* not found */
+    /* not found or from wrong node */
 
     if ((server_init_type == RECOV_COLD) ||
         (server_init_type == RECOV_CREATE)) 
@@ -1678,9 +1755,9 @@ void req_jobobit(
 
         pjob->ji_qs.ji_svrflags |= JOB_SVFLG_HASRUN|JOB_SVFLG_CHKPT;
 
-        svr_evaljobstate(pjob, &newstate, &newsubst, 1);
+        svr_evaljobstate(pjob,&newstate,&newsubst,1);
 
-        svr_setjobstate(pjob, newstate, newsubst);
+        svr_setjobstate(pjob,newstate,newsubst);
 
         svr_disconnect(pjob->ji_momhandle);
 
@@ -1753,6 +1830,11 @@ void req_jobobit(
 			
     ptask = set_task(WORK_Immed, 0, on_job_exit, (void *)pjob);
 
+    if (ptask)
+      {
+      append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
+      }
+
     /* "on_job_exit()" will be dispatched out of the main loop */
     } 
   else 
@@ -1781,11 +1863,18 @@ void req_jobobit(
       return;
       }
 
-    svr_setjobstate(pjob,JOB_STATE_EXITING, 
+    svr_setjobstate(
+      pjob,
+      JOB_STATE_EXITING, 
       pjob->ji_qs.ji_substate);
 
     ptask = set_task(WORK_Immed,0,on_job_rerun,(void *)pjob);
 
+    if (ptask)
+      {
+      append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
+      }
+
     /* "on_job_rerun()" will be dispatched out of the main loop */
     }
 
