@@ -108,6 +108,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <unistd.h>
 #include "server_limits.h"
 #include "list_link.h"
 #include "attribute.h"
@@ -130,7 +131,7 @@
 /* Global Data Items: */
 
 extern struct server server;
-extern list_head     svr_queues;
+extern tlist_head     svr_queues;
 extern attribute_def que_attr_def[];
 extern attribute_def svr_attr_def[];
 extern int  svr_chngNodesfile;
@@ -146,6 +147,8 @@ extern char *msg_man_uns;
 extern int que_purge(pbs_queue *);
 extern void save_characteristic(struct pbsnode *);
 extern int chk_characteristic(struct pbsnode *,int *);
+extern int hasprop(struct pbsnode *, struct prop *);
+extern int PNodeStateToString(int,char *,int);
 
 
 
@@ -382,9 +385,13 @@ static int mgr_set_attr(
         {
         if ((rc = (pdef + index)->at_action(new + index,parent,mode))) 
           {
-          attr_atomic_kill(new,pdef,limit);
+             /* always allow removing from ACLs */
+          if (!((plist->al_op == DECR) && (pdef + index)->at_type == ATR_TYPE_ACL))
+            {
+            attr_atomic_kill(new,pdef,limit);
 
-          return(rc);
+            return(rc);
+            }
           }
         }
 
@@ -569,86 +576,6 @@ static void update_subnode(
 
 
 		
-int __PNodeStateToString(
-
-  int   SBM,     /* I (state bitmap) */
-  char *Buf,     /* O */
-  int   BufSize) /* I */
-
-  {
-  if (Buf == NULL)
-    {
-    return(-1);
-    }
-
-  Buf[0] = '\0';
-
-  if (SBM & (INUSE_DOWN))
-    {
-    if (Buf[0] != '\0')
-      strcat(Buf,",");
-
-    strcat(Buf,ND_down);
-    }
-
-  if (SBM & (INUSE_OFFLINE))
-    {
-    if (Buf[0] != '\0')
-      strcat(Buf,",");
-
-    strcat(Buf,ND_offline);
-    }
-
-  if (SBM & (INUSE_JOB))
-    {
-    if (Buf[0] != '\0')
-      strcat(Buf,",");
-
-    strcat(Buf,ND_job_exclusive);
-    }
-
-  if (SBM & (INUSE_JOBSHARE))
-    {
-    if (Buf[0] != '\0')
-      strcat(Buf,",");
-
-    strcat(Buf,ND_job_sharing);
-    }
-
-  if (SBM & (INUSE_BUSY))
-    {
-    if (Buf[0] != '\0')
-      strcat(Buf,",");
-
-    strcat(Buf,ND_busy);
-    }
-
-  if (SBM & (INUSE_RESERVE))
-    {
-    if (Buf[0] != '\0')
-      strcat(Buf,",");
-
-    strcat(Buf,ND_reserve);
-    }
-
-  if (SBM & (INUSE_UNKNOWN))
-    {
-    if (Buf[0] != '\0')
-      strcat(Buf,",");
-
-    strcat(Buf,ND_state_unknown);
-    }
-
-  if (Buf[0] == '\0')
-    {
-    strcat(Buf,ND_free);
-    }
-
-  return(0);
-  }  /* END __PNodeStateToString() */
-
-
-
 
 
 /*
@@ -706,7 +633,7 @@ int mgr_set_node_attr(
 
   if (plist == NULL)
     {
-    return(0);		/*nothing to do, return success*/
+    return(0);		/* nothing to do, return success */
     }
 
   /* Get heap space for a temporary node-attribute array and use the
@@ -825,10 +752,10 @@ int mgr_set_node_attr(
  
     /* log change */
 
-    __PNodeStateToString(pnode->nd_state,OrigState,sizeof(OrigState));
-    __PNodeStateToString(tnode.nd_state,FinalState,sizeof(FinalState));
+    PNodeStateToString(pnode->nd_state,OrigState,sizeof(OrigState));
+    PNodeStateToString(tnode.nd_state,FinalState,sizeof(FinalState));
 
-    sprintf(log_buffer,"node %s state changed from %s to %s\n",
+    sprintf(log_buffer,"node %s state changed from %s to %s",
       pnode->nd_name,
       OrigState,
       FinalState);
@@ -1081,6 +1008,7 @@ void mgr_server_set(
   svrattrl *plist;
   int	    rc;
 
+
   plist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_manager.rq_attr);
 
   rc = mgr_set_attr(
@@ -1093,7 +1021,106 @@ void mgr_server_set(
     (void *)&server,
     ATR_ACTION_ALTER);
 
-  if (rc != 0)
+  /* PBSE_BADACLHOST - lets show the user the first bad host in the ACL  */
+  if (rc == PBSE_BADACLHOST)
+    {
+    char     *bad_host;
+    char     *host_entry;
+    char      hostname[PBS_MAXHOSTNAME + 1];
+    attribute temp;
+    int       index;
+    int       i;
+    struct    array_strings *pstr;
+    int       bhstrlen;
+
+    bhstrlen = PBS_MAXHOSTNAME + 17;
+    bad_host = malloc(sizeof(char) * (bhstrlen+1));
+
+    if (bad_host==NULL)
+      {
+      reply_badattr(PBSE_BADHOST, bad_attr, plist, preq);
+
+      return;
+      }
+
+    /* look into plist to find an offending attr */
+    while (plist != NULL)
+      {
+      /* only concerned about operators and managers */
+      if (strcmp(plist->al_name,ATTR_managers) && strcmp(plist->al_name,ATTR_operators))
+        {
+        plist = (svrattrl *)GET_NEXT(plist->al_link);
+        continue;
+        }
+
+      index = find_attr(svr_attr_def, plist->al_name, SRV_ATR_LAST); 
+
+      clear_attr(&temp, &svr_attr_def[index]);
+      svr_attr_def[index].at_decode(&temp,plist->al_name,plist->al_resc,plist->al_value); 
+
+      pstr = temp.at_val.at_arst;
+    
+      bad_host[0] = '\0';
+    
+      /* loop over all hosts in the request and perform same
+         check as manager_oper_chk*/
+      for (i = 0; i < pstr->as_usedptr; ++i)
+        {
+        host_entry = strchr(pstr->as_string[i], (int)'@');
+ 
+        /* if wildcard, we can't check */
+        if ((host_entry != NULL) && host_entry[1] != '*')
+          {
+          if (get_fullhostname(host_entry+1,hostname,PBS_MAXHOSTNAME) ||
+              strncmp(host_entry+1,hostname,PBS_MAXHOSTNAME))
+            {
+            snprintf(bad_host,bhstrlen,"First bad host: %s",host_entry+1);
+            break;
+            }
+          }
+        }
+
+      if (bad_host[0] != '\0')
+        break;
+
+      /* nothing wrong found in the request, let's try again
+         with the server's list */
+      pstr = server.sv_attr[(int)index].at_val.at_arst;
+
+      for (i = 0; i < pstr->as_usedptr; ++i)
+        {
+        host_entry = strchr(pstr->as_string[i], (int)'@');
+
+        /* if wildcard, we can't check */
+        if ((host_entry != NULL) && host_entry[1] != '*')
+          {
+          if (get_fullhostname(host_entry+1,hostname,PBS_MAXHOSTNAME) ||
+              strncmp(host_entry+1,hostname,PBS_MAXHOSTNAME))
+            {
+            snprintf(bad_host,bhstrlen,"First bad host: %s",host_entry+1);
+            break;
+            }
+          }
+        }
+
+      if (bad_host[0] != '\0')
+        break;
+
+      plist = (svrattrl *)GET_NEXT(plist->al_link);
+      }
+
+    if (bad_host[0] != '\0') /* we found a fully qualified host that was bad */
+      {
+      req_reject(PBSE_BADACLHOST, 0, preq, NULL, bad_host); 
+      }
+    else /* this shouldn't happen (return PBSE_BADACLHOST, but now we can't find the bad host) */
+      {
+      reply_badattr(PBSE_BADHOST, bad_attr, plist, preq);
+      }
+
+    return;
+    } /* end PBSE_BADACLHOST */
+  else if (rc != 0)
     {
     reply_badattr(rc,bad_attr,plist,preq);
  
@@ -1406,6 +1433,7 @@ void mgr_node_set(
   static int	need_todo = 0;
 
   int		allnodes = 0;
+  int		propnodes = 0;
   int		bad = 0;
   svrattrl	*plist;
   struct pbsnode  *pnode;
@@ -1416,18 +1444,33 @@ void mgr_node_set(
   int		problem_cnt = 0;
   char		*problem_names;
   struct pbsnode  **problem_nodes = NULL;
+  struct prop props;
 
   if ((*preq->rq_ind.rq_manager.rq_objname == '\0') ||
-      (*preq->rq_ind.rq_manager.rq_objname == '@'))    
+      (*preq->rq_ind.rq_manager.rq_objname == '@') ||
+      ((*preq->rq_ind.rq_manager.rq_objname == ':') &&   
+      (*(preq->rq_ind.rq_manager.rq_objname+1) != '\0')))    
     {
     /*In this instance the set node req is to apply to all */
     /*nodes at the local ('\0')  or specified ('@') server */
 
     if ((pbsndlist != NULL) && svr_totnodes) 
       {
-      nodename = all_nodes;
       pnode = *pbsndlist;
-      allnodes = 1;
+      if ((*preq->rq_ind.rq_manager.rq_objname == ':') && 
+          (strcmp(preq->rq_ind.rq_manager.rq_objname+1,"ALL") != 0))
+        {
+        propnodes = 1;
+        nodename = preq->rq_ind.rq_manager.rq_objname;
+        props.name = nodename+1;
+        props.mark = 1;
+        props.next = NULL;
+        }
+      else
+        {
+        allnodes = 1;
+        nodename = all_nodes;
+        }
       }
     else 
       { 
@@ -1458,7 +1501,7 @@ void mgr_node_set(
 
   log_event(PBSEVENT_ADMIN,PBS_EVENTCLASS_NODE,nodename,log_buffer);
 
-  if (allnodes) 
+  if (allnodes || propnodes) 
     {
     pnode = pbsndlist[0];
 
@@ -1471,6 +1514,9 @@ void mgr_node_set(
 
   for (i = 0;i < svr_totnodes;i++,pnode = pbsndlist[i]) 
     {
+    if (propnodes && !hasprop(pnode,&props))
+      continue;
+
     save_characteristic(pnode);
 
     rc = mgr_set_node_attr(
@@ -1485,7 +1531,7 @@ void mgr_node_set(
 
     if (rc != 0) 
       {
-      if (allnodes) 
+      if (allnodes || propnodes) 
         {
         if (problem_nodes)     /*we have an array in which to save*/
           problem_nodes[ problem_cnt ] = pnode;
@@ -1537,7 +1583,7 @@ void mgr_node_set(
         pnode->nd_name);
       }
 
-    if (!allnodes)
+    if (!allnodes && !propnodes)
       break;
     }  /* END for (i) */
 
@@ -1559,7 +1605,7 @@ void mgr_node_set(
     need_todo &= ~(WRITE_NEW_NODESFILE);	 /*successful on update*/
     }
 
-	if (allnodes) {          /*modification was for all nodes  */ 
+	if (allnodes || propnodes) {          /*modification was for all nodes  */ 
 
 	   if ( problem_cnt ) {  /*one or more problems encountered*/
 		
@@ -1866,7 +1912,7 @@ void mgr_node_create(
     PBS_EVENTCLASS_NODE, 
     preq->rq_ind.rq_manager.rq_objname);
 
-  setup_notification();	    /* set mechanism for notifying */
+  setup_notification(preq->rq_ind.rq_manager.rq_objname);	    /* set mechanism for notifying */
 			    /* other nodes of new member   */
 
   if (update_nodes_file()) 
@@ -2039,8 +2085,6 @@ void req_manager(
 
 
 
-
-
 /*
  * manager_oper_chk - check the @host part of a manager or operator acl
  *	entry to insure it is fully qualified.  This is to prevent
@@ -2101,13 +2145,20 @@ int manager_oper_chk(
           sprintf(log_buffer,"bad entry in acl: %s",
             pstr->as_string[i]);
 
-          log_err(PBSE_BADHOST, 
+          log_err(PBSE_BADACLHOST, 
             "manager_oper_chk",
             log_buffer);
           } 
         else 
           {
-          err = PBSE_BADHOST;
+          sprintf(log_buffer,"bad entry in acl: %s",
+            pstr->as_string[i]);
+
+          log_err(PBSE_BADACLHOST,
+            "manager_oper_chk",
+            log_buffer);
+
+          err = PBSE_BADACLHOST;
           }
         }
       }
@@ -2116,5 +2167,120 @@ int manager_oper_chk(
   return(err);
   }  /* END manager_oper_chk() */
 
+/*
+ * servername_chk - insure it is fully qualified.
+ *	This is the at_action() routine for the server attribute
+ *	"server_name"
+ */
+
+int servername_chk(
+
+  attribute *pattr,
+  void      *pobject,
+  int	     actmode)
+
+  {
+  int			 err = 0;
+  char			 hostname[PBS_MAXHOSTNAME + 1];
+  char *pstr;
+
+  extern char              server_name[];
+
+  if (actmode == ATR_ACTION_FREE)
+    {
+    if ((gethostname(hostname,PBS_MAXHOSTNAME) == -1) ||
+       (get_fullhostname(hostname,hostname,PBS_MAXHOSTNAME) == -1))
+      {
+      return(PBSE_BADHOST);
+      }
+
+    strcpy(server_name,hostname);
+
+    return(0);
+    }
+
+  pstr = pattr->at_val.at_str;
+
+  if (pstr == NULL)
+    {
+    return(PBSE_BADHOST);
+    }
+
+  /* must be fully qualified host */
+
+  if (get_fullhostname(pstr,hostname,PBS_MAXHOSTNAME) ||
+      strncmp(pstr,hostname,PBS_MAXHOSTNAME)) 
+    {
+    if (actmode == ATR_ACTION_RECOV) 
+      {
+      sprintf(log_buffer,"bad servername: %s",
+        pstr);
+
+      log_err(PBSE_BADHOST, 
+        "servername_chk",
+        log_buffer);
+      } 
+    else 
+      {
+      err = PBSE_BADHOST;
+      }
+    }
+  else
+    {
+    strcpy(server_name,hostname);
+    }
+
+  return(err);
+  }  /* END server_name_chk() */
+
+
+static int mgr_long_action_helper(
+
+  attribute *pattr,
+  int	     actmode,
+  long       minimum,
+  long       maximum)
+
+  {
+
+  int err = PBSE_NONE;
+
+  if (pattr->at_val.at_long < minimum)
+    {
+    err = PBSE_BADATVAL;
+    }
+
+  if (pattr->at_val.at_long > maximum)
+    {
+    err = PBSE_EXLIMIT;
+    }
+
+  if (err && (actmode == ATR_ACTION_RECOV)) 
+    {
+    pattr->at_val.at_long = maximum;
+    err = PBSE_NONE;
+    }
+
+  return(PBSE_NONE);
+  }  /* END mgr_long_action_helper() */
+
+/* END req_manager.c */
+
+/*
+ * schiter_chk - make sure values are sane
+ *	This is the at_action() routine for the server attribute
+ *	"scheduler_iteration"
+ */
+
+int schiter_chk(
+
+  attribute *pattr,
+  void      *pobject,
+  int	     actmode)
+
+  {
+  return(mgr_long_action_helper(pattr,actmode,1,PBS_SCHEDULE_CYCLE));
+  }  /* END schiter_chk() */
+
 /* END req_manager.c */
 
