@@ -126,10 +126,10 @@
 
 /* External Functions Called: */
 
-extern int  reply_jid A_((char *jobid));
+extern int  reply_jid A_((char *));
 extern void start_exec A_((job *));
 extern int  svr_authorize_jobreq A_((struct batch_request *,job *));
-extern int  svr_chkque A_((job *,pbs_queue *,char *,int));
+extern int  svr_chkque A_((job *,pbs_queue *,char *,int,char *));
 extern int  job_route A_((job *));
 
 /* Global Data Items: */
@@ -154,8 +154,8 @@ const char *TJobFileType[] = {
   NULL };
 
 extern int	 resc_access_perm;
-extern list_head svr_alljobs;
-extern list_head svr_newjobs;
+extern tlist_head svr_alljobs;
+extern tlist_head svr_newjobs;
 extern attribute_def job_attr_def[];
 extern char *path_jobs;
 extern char *pbs_o_host;
@@ -173,12 +173,12 @@ extern  char *msg_daemonname;
 
 /* Private Functions in this file */
 
-static job *locate_new_job A_((int sock, char *jobid));
+static job *locate_new_job A_((int,char *));
 
 #ifdef PNOT
-static int user_account_verify A_((char *arguser, char* argaccount));
-static char *user_account_default A_((char *arguser));
-static int user_account_read_user A_((char *arguser));
+static int user_account_verify A_((char *,char *));
+static char *user_account_default A_((char *));
+static int user_account_read_user A_((char *));
 #endif /* PNOT */
 
 #ifndef PBS_MOM
@@ -219,6 +219,7 @@ void req_quejob(
   pbs_queue	*pque;
   char		*qname;
   attribute	 tempattr;
+  char           EMsg[1024];
 #endif /* server */
 
   /* set basic (user) level access permission */
@@ -258,8 +259,14 @@ void req_quejob(
 
     created_here = JOB_SVFLG_HERE;
 
+#ifdef JOBARRAYTESTING
+    sprintf(jidbuf,"%d-%d.",
+      server.sv_qs.sv_jobidnumber,1);
+#else
     sprintf(jidbuf,"%d.",
       server.sv_qs.sv_jobidnumber);
+#endif
+
 
     strcat(jidbuf,server_name);
 
@@ -632,7 +639,7 @@ void req_quejob(
 
         return;
         }
-      }
+      }    /* END if (rc != 0) */
 
 #else	/* PBS_MOM */
 
@@ -759,7 +766,7 @@ void req_quejob(
 
     /* set create time */
 
-    pj->ji_wattr[(int)JOB_ATR_ctime].at_val.at_long =(long)time_now;
+    pj->ji_wattr[(int)JOB_ATR_ctime].at_val.at_long = (long)time_now;
     pj->ji_wattr[(int)JOB_ATR_ctime].at_flags |= ATR_VFLAG_SET;
 
     /* set hop count = 1 */
@@ -769,7 +776,8 @@ void req_quejob(
 
     /* Interactive jobs are necessarily not rerunable */
 
-    if (pj->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long)
+    if ((pj->ji_wattr[(int)JOB_ATR_interactive].at_flags & ATR_VFLAG_SET) &&
+        pj->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long)
       {
       pj->ji_wattr[(int)JOB_ATR_rerunable].at_val.at_long = 0;
       pj->ji_wattr[(int)JOB_ATR_rerunable].at_flags |= ATR_VFLAG_SET;
@@ -920,19 +928,21 @@ void req_quejob(
    * See if the job is qualified to go into the requested queue.
    * Note, if an execution queue, then ji_qs.ji_un.ji_exect is set up
    *
-   * svr_chkque is called way down here because it needs to have the
+   * svr_chkque() is called way down here because it needs to have the
    * job structure and attributes already set up.
    */
 
-  if ((rc = svr_chkque(pj,pque,preq->rq_host,MOVE_TYPE_Move))) 
+  if ((rc = svr_chkque(pj,pque,preq->rq_host,MOVE_TYPE_Move,EMsg))) 
     {
     job_purge(pj);
 
-    req_reject(rc,0,preq,NULL,NULL);
+    req_reject(rc,0,preq,NULL,EMsg);
 
     return;
     }
 
+  /* FIXME: if EMsg[0] != '\0', send a warning email to the user */
+
   strcpy(pj->ji_qs.ji_queue,pque->qu_qs.qu_name);
 
   pj->ji_wattr[(int)JOB_ATR_substate].at_val.at_long = JOB_SUBSTATE_TRANSIN;
@@ -1048,7 +1058,7 @@ void req_jobscript(
 
   errno = 0;
 
-  pj = locate_new_job(preq->rq_conn,NULL);
+  pj = locate_new_job(preq->rq_conn,preq->rq_ind.rq_jobfile.rq_jobid);
 
   if (pj == NULL) 
     {
@@ -1300,10 +1310,7 @@ void req_mvjobfile(  /* NOTE:  routine for server only - mom code follows this r
     }
 
   if (preq->rq_ind.rq_jobfile.rq_sequence == 0)
-    {
-    unlink(namebuf);
-    fds = open(namebuf,O_WRONLY|O_CREAT|O_EXCL|O_Sync,0600);
-    }
+    fds = open(namebuf,O_WRONLY|O_CREAT|O_TRUNC|O_Sync,0600);
   else
     fds = open(namebuf,O_WRONLY|O_APPEND|O_Sync,0600);
 
@@ -1859,6 +1866,9 @@ void req_commit(
  *	The job must (also) match the socket specified and the host associated
  *	with the socket unless ji_fromsock == -1, then its a recovery situation.
  */
+/* FIXME: why bother checking for matching sock if a jobid is supplied?  Seems
+ * to me that a reconnect immediately invalidates fromsock.
+ */
 
 static job *locate_new_job(
 
@@ -1876,7 +1886,7 @@ static job *locate_new_job(
        ((pj->ji_qs.ji_un.ji_newt.ji_fromsock == sock) &&
         (pj->ji_qs.ji_un.ji_newt.ji_fromaddr == get_connectaddr(sock)))) 
       {
-      if (jobid != NULL) 
+      if ((jobid != NULL) && (*jobid != '\0')) 
         {
         if (!strncmp(pj->ji_qs.ji_jobid,jobid,PBS_MAXSVRJOBID))
           {
