@@ -137,12 +137,12 @@ static void eval_chkpnt A_((attribute *j, attribute *q));
 
 extern struct server server;
 
-extern list_head svr_alljobs;
+extern tlist_head svr_alljobs;
 extern char  *msg_badwait;		/* error message */
 extern char  *msg_daemonname;
 extern char  *pbs_o_host;
 extern char   server_name[];
-extern list_head svr_queues;
+extern tlist_head svr_queues;
 extern int    comp_resc_lt;
 extern int    comp_resc_gt;
 extern int    svr_do_schedule;
@@ -272,7 +272,7 @@ int svr_enquejob(
     return(PBSE_UNKQUE);
     }
 
-  /* add job to server's all job list and update server counts */
+  /* add job to server's 'all job' list and update server counts */
 
 #ifndef NDEBUG
   sprintf(log_buffer, "enqueuing into %s, state %x hop %ld",
@@ -302,8 +302,7 @@ int svr_enquejob(
     {
     /* link first in server's list */
 
-    insert_link(&svr_alljobs, &pjob->ji_alljobs, pjob,
-      LINK_INSET_AFTER);
+    insert_link(&svr_alljobs,&pjob->ji_alljobs,pjob,LINK_INSET_AFTER);
     } 
   else 
     {
@@ -320,70 +319,82 @@ int svr_enquejob(
 
   pjob->ji_qhdr = pque;
 
-	pjcur = (job *)GET_PRIOR(pque->qu_jobs);
-	while (pjcur) {
-	    if ((unsigned long)pjob->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long >= (unsigned long)pjcur->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long)
-			break;
-		pjcur = (job *)GET_PRIOR(pjcur->ji_jobque);
-	}
-	if (pjcur == 0) {
-		/* link first in list */
-		insert_link(&pque->qu_jobs, &pjob->ji_jobque, pjob,
-			    LINK_INSET_AFTER);
-	} else {
-		/* link after 'current' job in list */
-		insert_link(&pjcur->ji_jobque, &pjob->ji_jobque, pjob,
-			    LINK_INSET_AFTER);
-	}
+  pjcur = (job *)GET_PRIOR(pque->qu_jobs);
 
-	/* update counts: queue and queue by state */
+  while (pjcur != NULL) 
+    {
+    if ((unsigned long)pjob->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long >= 
+        (unsigned long)pjcur->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long)
+      break;
 
-	pque->qu_numjobs++;
-	pque->qu_njstate[pjob->ji_qs.ji_state]++;
+    pjcur = (job *)GET_PRIOR(pjcur->ji_jobque);
+    }
 
-	/* update the current location and type attribute */
+  if (pjcur == NULL) 
+    {
+    /* link first in list */
 
-	pdef    = &job_attr_def[(int)JOB_ATR_in_queue];
-	pattrjb = &pjob->ji_wattr[(int)JOB_ATR_in_queue];
-	pdef->at_free(pattrjb);
-	pdef->at_decode(pattrjb, (char *)0, (char *)0, pque->qu_qs.qu_name);
+    insert_link(&pque->qu_jobs,&pjob->ji_jobque,pjob,LINK_INSET_AFTER);
+    } 
+  else 
+    {
+    /* link after 'current' job in list */
 
-	pjob->ji_wattr[(int)JOB_ATR_queuetype].at_val.at_char = 
-			*pque->qu_attr[(int)QA_ATR_QType].at_val.at_str;
-	pjob->ji_wattr[(int)JOB_ATR_queuetype].at_flags |= ATR_VFLAG_SET;
+    insert_link(&pjcur->ji_jobque,&pjob->ji_jobque,pjob,LINK_INSET_AFTER);
+    }
 
-	if ((pjob->ji_wattr[(int)JOB_ATR_qtime].at_flags & ATR_VFLAG_SET) ==0) {
-		pjob->ji_wattr[(int)JOB_ATR_qtime].at_val.at_long = time_now;
-		pjob->ji_wattr[(int)JOB_ATR_qtime].at_flags |= ATR_VFLAG_SET;
+  /* update counts: queue and queue by state */
 
-		/* issue enqueued accounting record */
+  pque->qu_numjobs++;
+  pque->qu_njstate[pjob->ji_qs.ji_state]++;
 
-		(void)sprintf(log_buffer, "queue=%s", pque->qu_qs.qu_name);
-		account_record(PBS_ACCT_QUEUE, pjob, log_buffer);
-	}
+  /* update the current location and type attribute */
+
+  pdef    = &job_attr_def[(int)JOB_ATR_in_queue];
+  pattrjb = &pjob->ji_wattr[(int)JOB_ATR_in_queue];
+  pdef->at_free(pattrjb);
+
+  pdef->at_decode(pattrjb,NULL,NULL,pque->qu_qs.qu_name);
+
+  pjob->ji_wattr[(int)JOB_ATR_queuetype].at_val.at_char = 
+    *pque->qu_attr[(int)QA_ATR_QType].at_val.at_str;
+
+  pjob->ji_wattr[(int)JOB_ATR_queuetype].at_flags |= ATR_VFLAG_SET;
+
+  if ((pjob->ji_wattr[(int)JOB_ATR_qtime].at_flags & ATR_VFLAG_SET) == 0) 
+    {
+    pjob->ji_wattr[(int)JOB_ATR_qtime].at_val.at_long = time_now;
+    pjob->ji_wattr[(int)JOB_ATR_qtime].at_flags |= ATR_VFLAG_SET;
+
+    /* issue enqueued accounting record */
+
+    sprintf(log_buffer,"queue=%s",
+      pque->qu_qs.qu_name);
+
+    account_record(PBS_ACCT_QUEUE,pjob,log_buffer);
+    }
 	
-	/*
-	 * set any "unspecified" resources which have default values,
-	 * first with queue defaults, then with server defaults
-	 */
+  /*
+   * set any "unspecified" resources which have default values,
+   * first with queue defaults, then with server defaults
+   */
 
-	set_resc_deflt(pjob);
+  set_resc_deflt(pjob,NULL);
 
-	/*
-	 * See if we need to do anything special based on type of queue
-	 */
+  /* See if we need to do anything special based on type of queue */
 
-	if (pque->qu_qs.qu_type == QTYPE_Execution) {
+  if (pque->qu_qs.qu_type == QTYPE_Execution) 
+    {
+    /* set union to "EXEC" and clear mom's address */
 
-		/* set union to "EXEC" and clear mom's address */
+    if (pjob->ji_qs.ji_un_type != JOB_UNION_TYPE_EXEC) 
+      {
+      pjob->ji_qs.ji_un_type = JOB_UNION_TYPE_EXEC;
+      pjob->ji_qs.ji_un.ji_exect.ji_momaddr = 0;
+      pjob->ji_qs.ji_un.ji_exect.ji_exitstat = 0;
+      }
 
-		if (pjob->ji_qs.ji_un_type != JOB_UNION_TYPE_EXEC) {
-			pjob->ji_qs.ji_un_type = JOB_UNION_TYPE_EXEC;
-			pjob->ji_qs.ji_un.ji_exect.ji_momaddr = 0;
-			pjob->ji_qs.ji_un.ji_exect.ji_exitstat = 0;
-		}
-
-    /* check the job checkpoint against the queue's  min */
+    /* check the job checkpoint against the queue's min */
 
     eval_chkpnt(
       &pjob->ji_wattr[(int)JOB_ATR_chkpnt],
@@ -398,7 +409,7 @@ int svr_enquejob(
              pjob,
              ATR_ACTION_NOOP)) != 0)
         {
-        return (rc);
+        return(rc);
         }
       }
 
@@ -469,6 +480,10 @@ void svr_dequejob(
 
       if (--pque->qu_njstate[pjob->ji_qs.ji_state] < 0)
         bad_ct = 1;
+
+      if (pjob->ji_qs.ji_state == JOB_STATE_COMPLETE)
+        if (--pque->qu_numcompleted < 0)
+          bad_ct = 1;
       }
 
     pjob->ji_qhdr = (pbs_queue *)0;
@@ -742,7 +757,8 @@ static void chk_svr_resc_limit(
   attribute *jobatr,
   attribute *queatr,
   attribute *svratr,
-  int	     qtype)  /* I */
+  int	     qtype,  /* I */
+  char      *EMsg)   /* O (optional,minsize=1024) */
 
   {
   int       dummy;
@@ -755,6 +771,9 @@ static void chk_svr_resc_limit(
   resource *svrc;
   resource *cmpwith;
 
+  int       LimitIsFromQueue;
+  char     *LimitName;
+
   static int SvrNodeCt = 0;
 
   static resource_def *noderesc = NULL;
@@ -763,6 +782,9 @@ static void chk_svr_resc_limit(
 
   /* NOTE:  server limits are specified with server.resources_available */
 
+  if (EMsg != NULL)
+    EMsg[0] = '\0';
+
   if (noderesc == NULL) 
     {
     noderesc = find_resc_def(svr_resc_def,"nodes",svr_resc_size);
@@ -808,6 +830,9 @@ static void chk_svr_resc_limit(
       {
       qurc = find_resc_entry(queatr,jbrc->rs_defin);
 
+      LimitIsFromQueue = 0;
+      LimitName = jbrc->rs_defin->rs_name;
+
       if ((qurc == NULL) || ((qurc->rs_value.at_flags & ATR_VFLAG_SET) == 0)) 
         {
         /* queue limit not set, check server's */
@@ -823,6 +848,8 @@ static void chk_svr_resc_limit(
         {
         /* queue limit is set, use it */
 
+        LimitIsFromQueue = 1;
+
         cmpwith = qurc;
         }
 
@@ -841,9 +868,20 @@ static void chk_svr_resc_limit(
                &jbrc->rs_value);
 
         if (rc > 0)
+          {
           comp_resc_gt++;
+          }
         else if (rc < 0)
+          {
+          if ((EMsg != NULL) && (EMsg[0] == '\0'))
+            {
+            sprintf(EMsg,"cannot satisfy %s max %s requirement",
+              (LimitIsFromQueue == 1) ? "queue" : "server",
+              (LimitName != NULL) ? LimitName : "resource");
+            }
+
           comp_resc_lt++;
+          }
         }
       }    /* END if () */
 
@@ -873,6 +911,9 @@ static void chk_svr_resc_limit(
 
       if ((SvrNodeCt > 0) && (tmpI <= SvrNodeCt))
         IgnTest = 1;
+
+      if (server.sv_attr[(int)SRV_ATR_NodePack].at_val.at_long)
+        IgnTest = 1;
       }
 
     if (IgnTest == 0)
@@ -884,6 +925,9 @@ static void chk_svr_resc_limit(
             &dummy,
             &dummy) == -1)
         {
+        if ((EMsg != NULL) && (EMsg[0] == '\0'))
+          strcpy(EMsg,"cannot locate feasible nodes");
+
         comp_resc_lt++;
         }
       }
@@ -905,16 +949,24 @@ static void chk_svr_resc_limit(
 int chk_resc_limits(
 
   attribute *pattr,  /* I */
-  pbs_queue *pque)   /* I */
+  pbs_queue *pque,   /* I */
+  char      *EMsg)   /* O (optional,minsize=1024) */
 
   {
   /* NOTE:  comp_resc_gt and comp_resc_lt are global ints */
 
   /* first check against queue minimum */
 
-  if ((comp_resc(&pque->qu_attr[QA_ATR_ResourceMin],pattr) == -1) ||
+  if ((comp_resc2(
+         &pque->qu_attr[QA_ATR_ResourceMin],
+         pattr,
+         server.sv_attr[(int)SRV_ATR_QCQLimits].at_val.at_long,
+         EMsg) == -1) ||
       (comp_resc_gt > 0))
     {
+    if ((EMsg != NULL) && (EMsg[0] == '\0'))
+      strcpy(EMsg,"job violates queue min resource limits");
+
     return(PBSE_EXCQRESC);
     }
 
@@ -924,13 +976,19 @@ int chk_resc_limits(
     pattr,
     &pque->qu_attr[QA_ATR_ResourceMax],
     &server.sv_attr[SRV_ATR_ResourceMax],
-    pque->qu_qs.qu_type);
+    pque->qu_qs.qu_type,
+    EMsg);
 
   if (comp_resc_lt > 0)
     {
+    if ((EMsg != NULL) && (EMsg[0] == '\0'))
+      strcpy(EMsg,"job violates queue/server max resource limits");
+
     return(PBSE_EXCQRESC);
     }
 
+  /* SUCCESS */
+
   return(0);
   }  /* END chk_resc_limits() */
 
@@ -953,12 +1011,20 @@ int svr_chkque(
   job	    *pjob,
   pbs_queue *pque,
   char	    *hostname,
-  int	     mtype)	/* MOVE_TYPE_* type, see server_limits.h */
+  int	     mtype,     /* MOVE_TYPE_* type, see server_limits.h */
+  char      *EMsg)      /* O (optional,minsize=1024) */
 
   {
   int i;
   int failed_group_acl = 0;
   int failed_user_acl  = 0;
+  int user_jobs;
+  job *pj;
+  struct array_strings *pas;
+  int j=0;
+
+  if (EMsg != NULL)
+    EMsg[0] = '\0';
 
   /*
    * 1. If the queue is an Execution queue ...
@@ -1006,14 +1072,18 @@ int svr_chkque(
     if (pque->qu_attr[QA_ATR_AclGroupEnabled].at_val.at_long)
       {
       int rc;
+      int slpygrp;
+
+      slpygrp = attr_ifelse_long(&pque->qu_attr[(int)QA_ATR_AclGroupSloppy],
+                                &server.sv_attr[(int)SRV_ATR_AclGroupSloppy],
+                                0);
 
       rc = acl_check(
          &pque->qu_attr[QA_ATR_AclGroup],
          pjob->ji_wattr[(int)JOB_ATR_egroup].at_val.at_str,
          ACL_Group);
 
-      if ((rc == 0) &&
-          (pque->qu_attr[QA_ATR_AclGroupSloppy].at_val.at_long))
+      if ((rc == 0) && slpygrp)
         {
         /* try again looking at the gids */
 
@@ -1023,8 +1093,7 @@ int svr_chkque(
            ACL_Gid);
         }
 
-      if ((rc == 0) &&
-          (pque->qu_attr[QA_ATR_AclGroupSloppy].at_val.at_long) &&
+      if ((rc == 0) && slpygrp &&
           (!(pjob->ji_wattr[(int)JOB_ATR_grouplst].at_flags & ATR_VFLAG_SET)))
         {
         /* check group acl against all accessible groups */
@@ -1036,36 +1105,27 @@ int svr_chkque(
 
         strncpy(uname,pjob->ji_wattr[(int)JOB_ATR_euser].at_val.at_str,PBS_MAXUSER);
 
-        setgrent();
+        /* fetch the groups in the ACL and look for matching user membership */
 
-        /* walk all groups looking for matching user membership */
+	pas = pque->qu_attr[QA_ATR_AclGroup].at_val.at_arst;
+	for (i=0;i<pas->as_usedptr;i++) 
+	  {
+          if ((grp = getgrnam(pas->as_string[i])) == NULL) 
+            continue;
 
-        while ((grp = getgrent()))
-          {
-          for (i = 0;grp->gr_mem[i] != NULL;i++)
+          for (j=0;grp->gr_mem[j] != NULL;j++) 
             {
-            if (strcmp(grp->gr_mem[i],uname))
-              continue;
-
-            rc = acl_check(
-              &pque->qu_attr[QA_ATR_AclGroup],
-              grp->gr_name,
-              ACL_Gid);
-
-            break;
+            if (!strcmp(grp->gr_mem[j],uname))
+              {
+              rc = 1;
+              break;
+              }
             }
 
-          if (rc != 0)
-            {
-            /* match found */
-
+          if (rc == 1)
             break;
-            }
-          }  /* END while (grp) */
-
-          endgrent();
-
-        }    /* END if (rc == 0) && AclGroupSloppy */
+          }
+        }    /* END if (rc == 0) && AclGroupSloppy...) */
 
       if (rc == 0)
         {
@@ -1073,8 +1133,13 @@ int svr_chkque(
 
         if (mtype != MOVE_TYPE_MgrMv) /* ok if mgr */
           {
-          if (pque->qu_attr[QA_ATR_AclLogic].at_val.at_long &&
-              pque->qu_attr[QA_ATR_AclUserEnabled].at_val.at_long)
+          int logic_or;
+
+          logic_or = attr_ifelse_long(&pque->qu_attr[(int)QA_ATR_AclLogic],
+                                      &server.sv_attr[(int)SRV_ATR_AclLogic],
+                                      0);
+              
+          if (logic_or && pque->qu_attr[QA_ATR_AclUserEnabled].at_val.at_long)
             {
             /* only fail if neither user nor group acls can be met */
 
@@ -1103,10 +1168,33 @@ int svr_chkque(
       }
 
     if ((pque->qu_attr[QA_ATR_MaxJobs].at_flags & ATR_VFLAG_SET) &&
-        (pque->qu_numjobs >= pque->qu_attr[QA_ATR_MaxJobs].at_val.at_long))
+        ((pque->qu_numjobs - pque->qu_numcompleted) >= pque->qu_attr[QA_ATR_MaxJobs].at_val.at_long))
       {
       return(PBSE_MAXQUED);
       }
+
+    if ((pque->qu_attr[QA_ATR_MaxUserJobs].at_flags & ATR_VFLAG_SET) &&
+        (pque->qu_attr[QA_ATR_MaxUserJobs].at_val.at_long >= 0))
+      {
+      /* count number of jobs user has in queue */
+      user_jobs = 0;
+      pj = (job *)GET_NEXT(pque->qu_jobs);
+      while (pj)
+        {
+        if ((pj->ji_qs.ji_state <= JOB_STATE_RUNNING) &&
+            (!strcmp(pj->ji_wattr[JOB_ATR_job_owner].at_val.at_str,
+                     pjob->ji_wattr[JOB_ATR_job_owner].at_val.at_str)))
+          {
+          user_jobs++;
+          }
+        pj = (job *)GET_NEXT(pj->ji_jobque);
+        }
+
+      if (user_jobs >= pque->qu_attr[QA_ATR_MaxUserJobs].at_val.at_long)
+        {
+        return(PBSE_MAXUSERQUED);
+        }
+      }
     
     /* 3. if "from_route_only" is true, only local route allowed */
 
@@ -1143,8 +1231,13 @@ int svr_chkque(
            pjob->ji_wattr[(int)JOB_ATR_job_owner].at_val.at_str,
            ACL_User) == 0)
         {
-        if (pque->qu_attr[QA_ATR_AclLogic].at_val.at_long &&
-            pque->qu_attr[QA_ATR_AclGroupEnabled].at_val.at_long)
+        int logic_or;
+        
+        logic_or = attr_ifelse_long(&pque->qu_attr[(int)QA_ATR_AclLogic],
+                                    &server.sv_attr[(int)SRV_ATR_AclLogic],
+                                    0);
+
+        if (logic_or && pque->qu_attr[QA_ATR_AclGroupEnabled].at_val.at_long)
           {
           /* only fail if neither user nor group acls can be met */
 
@@ -1168,13 +1261,17 @@ int svr_chkque(
 
     /* 6. resources of the job must be in the limits of the queue */
 
-    if ((i = chk_resc_limits(&pjob->ji_wattr[(int)JOB_ATR_resource],pque)) != 0)
+    if ((i = chk_resc_limits(&pjob->ji_wattr[(int)JOB_ATR_resource],pque,EMsg)) != 0)
       {
+      /* FAILURE */
+
       return(i);
       }
     }    /* END if (mtype != MOVE_TYPE_MgrMv) */
 
-  return(0);	/* all ok, job can enter queue */
+  /* SUCCESS - job can enter queue */
+
+  return(0);
   }  /* END svr_chkque() */
 
 
@@ -1341,6 +1438,17 @@ static void default_std(
   while (isdigit((int)*pd))       
     *(to + len++) = *pd++;
 
+#ifdef JOBARRAYTESTING
+  if (isdigit((int)*(pd+1)))
+    {
+    /* add the jatid (job array taskid) */
+    *(to + len++) = *pd++;
+
+    while (isdigit((int)*pd))       
+      *(to + len++) = *pd++;
+    }
+#endif
+
   *(to + len) = '\0';
 
   return;
@@ -1464,7 +1572,7 @@ static void set_deflt_resc(
                   &prescdt->rs_value,
                   SET) == 0)
               {
-              prescjb->rs_value.at_flags |= (ATR_VFLAG_SET|ATR_VFLAG_DEFLT);
+              prescjb->rs_value.at_flags |= (ATR_VFLAG_SET|ATR_VFLAG_DEFLT|ATR_VFLAG_MODIFY);
               }
             }
           }
@@ -1480,35 +1588,63 @@ static void set_deflt_resc(
 
 
 
+/* NOTE:  if ji_wattr parameter is passed in, update it */
+
 void set_resc_deflt(
 
-  job *pjob)
+  job       *pjob,     /* I (modified) */
+  attribute *ji_wattr) /* I (optional) decoded attributes  */
 
   {
+  attribute *ja;
+
   pbs_queue *pque;
+  attribute_def *pdef;
+  int i,rc;
 
   pque = pjob->ji_qhdr;
 
   assert(pque != NULL);
 
-  set_deflt_resc(
-    &pjob->ji_wattr[(int)JOB_ATR_resource],
-    &pque->qu_attr[(int)QA_ATR_ResourceDefault]);
+  if (ji_wattr != NULL)
+    ja = &ji_wattr[(int)JOB_ATR_resource];
+  else
+    ja = &pjob->ji_wattr[(int)JOB_ATR_resource];
 
-  set_deflt_resc(
-    &pjob->ji_wattr[(int)JOB_ATR_resource],
-    &server.sv_attr[(int)SRV_ATR_resource_deflt]);
+  /* apply queue defaults first since they take precedence */
 
-  set_deflt_resc(
-    &pjob->ji_wattr[(int)JOB_ATR_resource],
-    &pque->qu_attr[(int)QA_ATR_ResourceMax]);
+  set_deflt_resc(ja,&pque->qu_attr[(int)QA_ATR_ResourceDefault]);
 
-  set_deflt_resc(
-    &pjob->ji_wattr[(int)JOB_ATR_resource],
-    &server.sv_attr[(int)SRV_ATR_ResourceMax]);
+  /* server defaults will only be applied to attributes which have
+     not yet been set */
+
+  set_deflt_resc(ja,&server.sv_attr[(int)SRV_ATR_resource_deflt]);
+
+  /* apply queue max limits first since they take precedence */
+
+  set_deflt_resc(ja,&pque->qu_attr[(int)QA_ATR_ResourceMax]);
+
+  /* server max limits will only be applied to attributes which have
+     not yet been set */
+
+  set_deflt_resc(ja,&server.sv_attr[(int)SRV_ATR_ResourceMax]);
+
+  for (i = 0;i < JOB_ATR_LAST;++i)
+    {
+    pdef = &job_attr_def[i];
+
+    if ((pjob->ji_wattr[i].at_flags & ATR_VFLAG_SET) && (pdef->at_action))
+      {
+      if ((rc = pdef->at_action(&pjob->ji_wattr[i],pjob,ATR_ACTION_NEW)) != 0)
+        {
+        /* FIXME: return an actual error */
+        return;
+        }
+      }
+    }
 
   return;
-  }
+  }  /* END set_resc_deflt() */
 
 
 
@@ -1624,9 +1760,10 @@ static void correct_ct(
     {
     pc = log_buffer + strlen(log_buffer);
 
-    sprintf(pc,"; queue %s %d: ", 
+    sprintf(pc,"; queue %s %d (completed: %d): ", 
       pqj->qu_qs.qu_name,
-      pqj->qu_numjobs);
+      pqj->qu_numjobs,
+      pqj->qu_numcompleted);
 
     for (i = 0;i < PBS_NUMJOBSTATE;++i) 
       {
@@ -1644,6 +1781,7 @@ static void correct_ct(
        pque = (pbs_queue *)GET_NEXT(pque->qu_link)) 
     {
     pque->qu_numjobs = 0;
+    pque->qu_numcompleted = 0;
 
     for (i = 0;i < PBS_NUMJOBSTATE;++i)
       pque->qu_njstate[i] = 0;
@@ -1658,6 +1796,10 @@ static void correct_ct(
 
     (pjob->ji_qhdr)->qu_numjobs++;
     (pjob->ji_qhdr)->qu_njstate[pjob->ji_qs.ji_state]++;
+
+    if (pjob->ji_qs.ji_state == JOB_STATE_COMPLETE)
+      pque->qu_numcompleted++;
+
     }  /* END for (pjob) */
 
   return;
