@@ -202,7 +202,7 @@ int decode_rcost(patr, name, rescn, val)
 
 int encode_rcost(attr, phead, atname, rsname, mode)
 	attribute	*attr;	  /* ptr to attribute */
-	list_head	*phead;	  /* head of attrlist list */
+	tlist_head	*phead;	  /* head of attrlist list */
 	char		*atname;  /* attribute name */
 	char		*rsname;  /* resource name or null */
 	int		mode;	  /* encode mode, unused here */
